import os,sys
import logging
import optparse
import subprocess
import numpy as np
import shutil
import warnings
import matplotlib; matplotlib.use('Agg')
import matplotlib.pyplot as plt
import matplotlib.cm as cm
import random
import bhtsne

class Features:
	def __init__( self, opts, mfeats_fn, ofeats_fn ):
		self.opts = opts
		self.mfn  = mfeats_fn
		self.ofn  = ofeats_fn

		if self.mfn.find("contig_methyl_features.txt")>-1:
			self.opts.seq_type = "contig"
		else:
			self.opts.seq_type = "read"

		self.parse_fields()

	def parse_fields( self ):
		"""
		Read in the fields contained in the output files from methylprofiles.
		"""
		m = np.loadtxt(self.mfn, dtype="str", skiprows=1)
		o = np.loadtxt(self.ofn, dtype="str", skiprows=1)
		
		# Optional flags
		m = self.length_filter(m)
		o = self.length_filter(o)
		if self.opts.n_seqs!=None:
			m = self.subsample_seqs( m )
			o = self.subsample_seqs( o )
		
		# Pull out values
		self.ids     = m[:,0].astype("str")
		self.lens    = m[:,1].astype("int")
		self.mscores = m[:,2:].astype("float")
		if self.opts.seq_type=="contig":
			self.covs     = o[:,2].astype("float")
			self.covcomps = o[:,2:].astype("float")
			self.comps    = o[:,3:].astype("float")
		else:
			self.comps    = o[:,2:].astype("float")

	def length_filter ( self, data ):
		"""
		Remove all sequences shorter than --l_min
		"""
		mk = data[:,1].astype("int")>self.opts.l_min

		return data[mk,:]

	def subsample_seqs( self, data ):
		"""
		If --n_seqs specified, downsample to desired number of sequences.
		"""
		mk = random.sample(range(data.shape[0]), self.opts.n_seqs)
		
		return data[mk,:]

	def tsne_reduce( self, data ):
		"""
		Use Barnes-Hut approximation of t-SNE to reduce dimensionality
		of features from N-dim to 2D for plotting.
		"""
		map2d = bhtsne.run_bh_tsne(data, no_dims=self.opts.n_dims, max_iter=self.opts.n_iters, use_pca=False, verbose=True)

		return map2d

	def PCA_reduce( self ):
		"""
		Use principal component analysis to reduce dimensionality
		of features from N-dim to 2D for plotting.
		"""
		# TO BE ADDED

	def plot( self, map2d, plot_fn ):
		"""
		Make scatter plot of 2-dimensional map.
		"""
		def scale_sizes(lengths):
			# Scale marker sizes for plotting
			max_size     = 2500
			min_size     = 5
			norm_lengths = (lengths-min(lengths)) / float((max(lengths)-min(lengths)))
			scaled_sizes = max_size * norm_lengths + min_size
			
			return scaled_sizes

		if self.opts.labels!=None:
			# Read labels into dictionary
			seq_labels_m       = np.loadtxt(self.opts.labels, dtype="str", delimiter="\t")
			seq_names          = seq_labels_m[:,0]
			seq_labs           = seq_labels_m[:,1]
			lab_d              = dict( zip(seq_names, seq_labs) )

			fig                = plt.figure(figsize=[15,12])
			ax                 = fig.add_axes([0.1, 0.1, 0.6, 0.6])
			# ax.axis("off")
			label_list         = list(set(lab_d.values()))
			label_list.sort()
			colors             = plt.get_cmap('spectral')(np.linspace(0.0, 0.95, len(label_list)))
			shapes             = ["o", "v", "^", "s", "D"]
			
			if self.opts.size_markers:
				scaled_sizes = scale_sizes(self.lens)

			res                = []
			for k,target_lab in enumerate(label_list):
				idxs  = [j for j,label in enumerate(seq_labs) if label==target_lab]
				X     = map2d[idxs,0]
				Y     = map2d[idxs,1]
				color = colors[k]
				if self.opts.size_markers:
					scaled_sizes_idx = scaled_sizes[idxs]
					for i,x in enumerate(X):
						res.append( (x, Y[i], target_lab, color, scaled_sizes_idx[i], shapes[k%len(shapes)]) )
				else:
					for i,x in enumerate(X):
						m_size = 15
						res.append( (x, Y[i], target_lab, color, m_size, shapes[k%len(shapes)]) )

			np.random.shuffle(res)
			plotted_labs = set()
			legend_plots = []
			legend_labs  = []
			for i,(x,y, target_lab, color, size, shape) in enumerate(res):
				if target_lab=="Unlabeled":
					shape = "*"
					color = "gray"
				plot = ax.scatter(x,y, marker=shape, s=size ,edgecolors=color, label=target_lab, facecolors="None", linewidth=1.2, alpha=0.8)
				if target_lab not in plotted_labs:
					plotted_labs.add(target_lab)
					legend_plots.append(plot)
					legend_labs.append(target_lab)
			box     = ax.get_position()
			ax.set_position([box.x0, box.y0, box.width * 0.7, box.height])
			leg_tup = zip(legend_labs,legend_plots)
			leg_tup.sort(key=lambda x: x[0])

			# Put "Unlabeled" at end of legend 
			unlabeled = [tup for tup in leg_tup if tup[0]=="Unlabeled"]
			if len(unlabeled)>0:
				unlabeled = unlabeled[0]
				leg_tup.remove(unlabeled)
				leg_tup.append(unlabeled)

			legend_labs, legend_plots = zip(*leg_tup)
			leg = ax.legend(legend_plots, legend_labs, loc='center left', prop={'size':24}, ncol=1, bbox_to_anchor=(1, 0.5), frameon=False, scatterpoints=1)
			for i in range(len(legend_labs)):
				leg.legendHandles[i]._sizes = [100]
			xmin  = min(map2d[:,0])
			xmax  = max(map2d[:,0])
			ymin  = min(map2d[:,1])
			ymax  = max(map2d[:,1])
			xspan = xmax - xmin
			yspan = ymax - ymin
			ax.set_xlim([xmin-(0.05*xspan), xmax+(0.05*xspan)])
			ax.set_ylim([ymin-(0.05*yspan), ymax+(0.05*yspan)])
			plt.savefig(plot_fn)
		else:
			fig = plt.figure(figsize=[12,12])
			ax  = fig.add_subplot(111)
			X   = map2d[:,0]
			Y   = map2d[:,1]
			if self.opts.size_markers:
				scaled_sizes = scale_sizes(self.lens)
				ax.scatter(X, Y, marker="o", lw=3, alpha=0.5, s=scaled_sizes)
			else:
				ax.scatter(X, Y, marker="o", s=15 ,edgecolors="None")
			xmin  = min(map2d[:,0])
			xmax  = max(map2d[:,0])
			ymin  = min(map2d[:,1])
			ymax  = max(map2d[:,1])
			xspan = xmax - xmin
			yspan = ymax - ymin
			ax.set_xlim([xmin-(0.05*xspan), xmax+(0.05*xspan)])
			ax.set_ylim([ymin-(0.05*yspan), ymax+(0.05*yspan)])
			logging.info("  * Saving %s map %s" % (self.opts.dim_reduce, plot_fn))
			plt.savefig(plot_fn)

	def digest_large_seqs( self ):
		"""
		Digests large sequences (probably just contigs) to give them 
		additional weight during t-SNE dimensionality reduction. Otherwise
		a large 3 Mb contig would have the same weight as a tiny 5 kb contig.
		"""
		max_size        = 50000
		self.lens_d     = []
		self.ids_d      = []
		self.mscores_d  = []
		self.comps_d    = []
		self.covs_d     = []
		self.covcomps_d = []
		for i,l in enumerate(self.lens):
		
			# Add original vals for all sequences
			self.lens_d.append(max_size)
			self.ids_d.append(self.ids[i])
			self.mscores_d.append(self.mscores[i,:])
			self.comps_d.append(self.comps[i,:])
			if self.opts.seq_type=="contig":
				self.covs_d.append(self.covs[i])
				self.covcomps_d.append(self.covcomps[i,:])
			
			# For large seqs, add values for digested seqs
			if l>max_size:
				j = 0
				k = 0
				while j < (l-max_size):
					j+=max_size
					self.lens_d.append(max_size)
					self.ids_d.append(self.ids[i]+".digest.%s" % k)
					self.mscores_d.append(self.mscores[i,:])
					self.comps_d.append(self.comps[i,:])
					if self.opts.seq_type=="contig":
						self.covs_d.append(self.covs[i])
						self.covcomps_d.append(self.covcomps[i,:])
					k+=1

		self.lens_d     = np.asarray(self.lens_d)
		self.ids_d      = np.asarray(self.ids_d)
		self.mscores_d  = np.asarray(self.mscores_d)
		self.comps_d    = np.asarray(self.comps_d)
		self.covs_d     = np.asarray(self.comps_d)
		self.covcomps_d = np.asarray(self.covs_d)

	def remove_digested_seqs( self, coords_d_2d ):
		"""
		After the digested sequences have been used to give
		large contigs more weight during t-SNE dimensionality
		reduction, remove those digested contigs so that only
		the original contigs are shown in the plot.
		"""
		mk              = np.ones(len(self.ids_d), dtype=bool) - 1
		digest_idx      = np.where(np.char.find(self.ids_d, '.digest.') < 0)
		mk[digest_idx] += 1
		mk              = mk.astype("bool")
		
		return coords_d_2d[mk]

def launch():
	opts,mfeats_fn,ofeats_fn = __parseArgs()
	__initLog(opts)

	feats = Features(opts, mfeats_fn, ofeats_fn)

	if feats.opts.dim_reduce=="bhtsne":
		# First digest any large contigs to give them
		# more weight during t-SNE clustering
		logging.info("Processing large contigs prior to BH-tSNE")
		feats.digest_large_seqs()
		logging.info("Using BH-tSNE to reduce methylation scores matrix from %s to %s" % (feats.mscores.shape[1], \
					 																	  feats.opts.n_dims))
		mscores2d_d  = feats.tsne_reduce(feats.mscores_d)
		mscores2d    = feats.remove_digested_seqs( mscores2d_d )
		mscores2d_fn = feats.mfn.replace(".txt", ".%s.txt" % feats.opts.dim_reduce)
		np.savetxt(mscores2d_fn, mscores2d, delimiter="\t", fmt="%.5f")
		plot_fn      = feats.mfn.replace(".txt", ".%s.png" % feats.opts.dim_reduce)
		if feats.opts.n_dims==2:
			feats.plot(mscores2d, plot_fn)
		print ""

		logging.info("Using BH-tSNE to reduce methylation scores matrix from %s to %s" % (feats.comps.shape[1], \
					 																	  feats.opts.n_dims))
		comps2d_d  = feats.tsne_reduce(feats.comps_d)
		comps2d    = feats.remove_digested_seqs( comps2d_d )
		comps2d_fn = feats.ofn.replace(".txt", ".comp.%s.txt" % feats.opts.dim_reduce)
		np.savetxt(comps2d_fn, comps2d, delimiter="\t", fmt="%.5f")
		plot_fn    = feats.ofn.replace(".txt", ".comp.%s.png" % feats.opts.dim_reduce)
		if feats.opts.n_dims==2:
			feats.plot(comps2d, plot_fn)
		print ""

		if feats.opts.seq_type=="contig":
			logging.info("Using BH-tSNE to reduce composition matrix (+ coverage column) from %s to %s" % (feats.covcomps.shape[1], \
						 																				   feats.opts.n_dims))
			covcomps2d_d  = feats.tsne_reduce(feats.covcomps_d)
			covcomps2d    = feats.remove_digested_seqs( covcomps2d_d )
			covcomps2d_fn = feats.ofn.replace(".txt", ".covcomp.%s.txt" % feats.opts.dim_reduce)
			np.savetxt(covcomps2d_fn, covcomps2d, delimiter="\t", fmt="%.5f")
			plot_fn       = feats.ofn.replace(".txt", ".covcomp.%s.png" % feats.opts.dim_reduce)
			if feats.opts.n_dims==2:
				feats.plot(covcomps2d, plot_fn)

	elif feats.opts.dim_reduce=="pca":
		raise Exception("PCA not yet supported. Sorry!")

	print >> sys.stderr, "mBin feature mapping has finished running. See log for details."

def __parseArgs():
	"""Handle command line argument parsing"""

	usage = """%prog [--help] [options] <SEQ>_methyl_features.txt <SEQ>_other_features.txt

	mapfeatures visualizes the landscape of high-dimensional sequence features 
	using the Barnes Hut approximation of t-SNE (PCA support coming soon). The 
	sequence features that are output from methylprofiles are often high-
	dimensional (>3D), making it difficult to visualize the sequences. To ease 
	this visualization for resolution of discrete sequence clusters in the feature 
	space, t-SNE is used to reduce the dimensionality of the methylation, 
	composition, and coverage features to 2D. The resulting 2D maps, which can be 
	overlaid with sequence annotation labels (generated with Kraken, for instance), 
	often reveals sequence clustering in the 2D feature space representing distinct
	taxonomical groups for binning.

	Two files from methylprofiles serve as input to mapfeatures: 
	   (1) <prefix>_<seq>_methyl_features.txt
	   (2) <prefix>_<seq>_other_features.txt 

	<prefix> is defined by --prefix and <seq> is the sequence data type: contig, align, 
	or read.


	Examples:

	### Plot contigs using supplied labels and make glyph size proportional to contig size ###

	mapfeatures -i --labels=<LABELS.txt> --size_markers contig_methyl_features.txt contig_other_features.txt


	### Plot aligned reads (longer than 500bp) using the supplied labels ###

	mapfeatures -i --labels=<LABELS.txt> --l_min=500 align_methyl_features.txt align_other_features.txt


	### Plot N=1000 unaligned reads longer than 10,000bp ###

	mapfeatures -i --l_min=10000 --n_seqs=1000 read_methyl_features.txt read_other_features.txt
	"""

	parser = optparse.OptionParser( usage=usage, description=__doc__ )

	parser.add_option( "-d", "--debug", action="store_true", help="Increase verbosity of logging" )

	parser.add_option( "-i", "--info", action="store_true", help="Add basic logging" )

	parser.add_option( "--logFile", type="str", help="Write logging to file [log.controls]" )
	
	parser.add_option( "--prefix", type="str", help="Prefix to use for output files [None]" )

	parser.add_option( "--size_markers", action="store_true", help="Adjust marker size in plot according to sequence length [False]" )
	
	parser.add_option( "--dim_reduce", type="str", help="Dimensionality reduction algorithm to apply (bhtsne or pca) [bhtsne]" )
	
	parser.add_option( "--labels", type="str", help="Tab-delimited file (no header) of sequence labels (seq_name\tlabel_name) [None]" )

	parser.add_option( "--l_min", type="int", help="Minimum read length to include for analysis [0]" )

	parser.add_option( "--n_seqs", type="int", help="Number of sequences to subsample [all]" )
	
	parser.add_option( "--n_dims", type="int", help="Number of dimensions to reduce to for visualization (only n_dims=2 will be plotted) [2]" )
	
	parser.add_option( "--n_iters", type="int", help="Number of iterations to use for BH-tSNE [500]" )
	
	parser.set_defaults( logFile="log.mapfeatures",          \
						 info=False,                         \
						 debug=False,                        \
						 prefix="",                          \
						 size_markers=False,                 \
						 dim_reduce="bhtsne",                \
						 labels=None,                        \
						 l_min=0,                            \
						 n_seqs=None,                        \
						 n_dims=2,                           \
						 n_iters=500,                        \
						 )

	opts, args = parser.parse_args( )

	mfeats_fn,ofeats_fn = __check_input( opts, args, parser )

	opts.comp_only     = False
	opts.sam           = None
	opts.skip_motifs   = None
	opts.bas_whitelist = None

	if opts.prefix != "":
		opts.prefix+="_"		
	
	return opts,mfeats_fn,ofeats_fn

def __initLog( opts ):
	"""Sets up logging based on command line arguments. Allows for three levels of logging:
	logging.error( ): always emitted
	logging.info( ) : emitted with --info or --debug
	logging.debug( ): only with --debug"""

	if os.path.exists(opts.logFile):
		os.remove(opts.logFile)

	logLevel = logging.DEBUG if opts.debug \
				else logging.INFO if opts.info \
				else logging.ERROR

	logger = logging.getLogger("")
	logger.setLevel(logLevel)
	
	# create file handler which logs even debug messages
	fh = logging.FileHandler(opts.logFile)
	fh.setLevel(logLevel)
	
	# create console handler with a higher log level
	ch = logging.StreamHandler()
	ch.setLevel(logLevel)
	
	# create formatter and add it to the handlers
	logFormat = "%(asctime)s [%(levelname)s] %(message)s"
	formatter = logging.Formatter(logFormat, "%Y-%m-%d %H:%M:%S")
	ch.setFormatter(formatter)
	fh.setFormatter(formatter)
	
	# add the handlers to logger
	logger.addHandler(ch)
	logger.addHandler(fh)

def __check_input( opts, args, parser ):
	"""
	Make sure the input consists of both files containing the 
	features assessed using buildprofiles:
	   <SEQ>_methyl_features.txt
	   <SEQ>_other_features.txt
	"""
	if len(args)!=2:
		print "ERROR -- expecting two arguments:"
		print "  (1) <SEQ>_methyl_features.txt output from methylprofiles containing methylation features for mapping"
		print "  (2) <SEQ>_other_features.txt output from methylprofiles containing alternative sequence features for mapping"
		print ""
		sys.exit()

	mfeats_fn  = args[0]
	ofeats_fn  = args[1]
	feature_type = None

	if not os.path.exists(mfeats_fn):
		parser.error("Can't find file of sequence features (methylprofiles output) for mapping: %s" % mfeats_fn)

	if not os.path.exists(ofeats_fn):
		parser.error("Can't find file of sequence features (methylprofiles output) for mapping: %s" % ofeats_fn)

	return mfeats_fn, ofeats_fn

if __name__ == "__main__":
	main()


import os,sys
import optparse
import logging
from pbcore.io.align.CmpH5IO import CmpH5Reader
from pbcore.io import openIndexedAlignmentFile
from pbcore.io.BasH5IO import BasH5Reader
import glob
import numpy as np
import logging
import shutil
import pickle
import math
import mbin
import motif_tools

def launch():
	opts,control_aln_fn = __parseArgs()
	__initLog(opts)

	extract_controls(opts, control_aln_fn)

	print >> sys.stderr, "mBin control extraction has finished running. See log for details."

def extract_controls(opts, control_aln_fn):
	"""

	"""
	controls       = ControlRunner(control_aln_fn, opts)
	mbinRunner     = mbin.mbinRunner(opts)

	# Pulling the IPD data for each motif from the WGA cmp.h5 file
	motifs,bi_motifs = motif_tools.build_motif_dict(opts)
	opts.motifs      = motifs
	opts.bi_motifs   = bi_motifs

	logging.info("")
	logging.info("Preparing to create new control data in %s" % opts.control_tmp)
	controls.goto_control_output_dir()
	
	opts           = controls.scan_WGA_aligns()
	filter_N_reads = opts.N_reads

	mbinRunner.launch_data_loader( control_aln_fn, filter_N_reads, 1, opts )
	
	controls.analyze_WGA_reads()
	logging.info("Done.")
	logging.info("")

	# Building dictionary of mean control IPD values for each motif
	logging.info("Building dictionary of control values for all motifs...")
	logging.info("   * Initial build requires significant time and memory.")
	controls.combine_control_data_from_contigs()

	
	control_means    = controls.build_control_IPD_dict(motifs, bi_motifs)
	controls.return_to_orig_dir()

	logging.info("")
	logging.info("Cleaning up temp files from control data processing...")
	shutil.rmtree(opts.control_tmp)

	# Controls are loaded into control_means, now pickle them for easy
	# passing between parallel processes
	pickle.dump(control_means, open(opts.control_pkl_name, "wb"))

def chunks( l, n ):
	"""
	Yield successive n-sized chunks from l.
	"""
	for i in xrange(0, len(l), n):
		yield l[i:i+n]

def __parseArgs():
	"""Handle command line argument parsing"""

	usage = """%prog [--help] [options] [input]

	buildcontrols takes a set of whole-genome amplified (WGA) sequencing 
	alignments, either in the legacy *.cmp.h5 PacBio format or the newer
	aligned BAM format. buildcontrols then reads the polymerase kinetics
	values from these alignments and constructs a pickled dictionary of
	control IPD values for the specified set of motifs. This output is 
	written to ./control_ipds.pkl by default.

	Examples:

	### Get control IPD values from BAM file of aligned WGA reads ###

	buildcontrols -i --ref=reference.fasta wga_aligned_reads.bam

	
	### Use 4 cores to process legacy cmp.h5 file of aligned WGA reads ###

	buildcontrols -i --procs=4 wga_aligned_reads.cmp.h5

	
	### Only include data from the first 500 alignments (e.g. for testing) ###

	buildcontrols -i --N_reads=500 --ref=reference.fasta wga_aligned_reads.bam


	### Exclude all bipartite motifs from the control dictionary ###

	buildcontrols -i --no_bipartite --ref=reference.fasta wga_aligned_reads.bam
	"""

	parser = optparse.OptionParser( usage=usage, description=__doc__ )

	parser.add_option( "-d", "--debug", action="store_true", help="Increase verbosity of logging" )

	parser.add_option( "-i", "--info", action="store_true", help="Add basic logging" )

	parser.add_option( "--logFile", type="str", help="Write logging to file [log.controls]" )

	parser.add_option( "--ref", type="str", help="Path to reference fasta file used in the alignment. Must be accompanied by an index file (use samtools faidx). Required if input is BAM file of aligned reads (not needed for cmp.h5). [None]" )
	
	parser.add_option( "--subreadlength_min", type="int", help="Minimum subread length to include for analysis [100]" )

	parser.add_option( "--readlength_min", type="int", help="Minimum read length to include for analysis [100]" )

	parser.add_option( "--min_kmer", type="int", help="Minimum motif size to scan (contiguous motifs) [4]" )

	parser.add_option( "--max_kmer", type="int", help="Maximum motif size to scan (contiguous motifs) [6]" )

	parser.add_option( "--no_bipartite", action="store_true", help="Omit bipartite motifs [False]" )
	
	parser.add_option( "--bipart_first", type="str", help="Bipartite motif configuration: acceptable length of first determinate component (comma-separated string of integers) [3,4]" )
	
	parser.add_option( "--bipart_Ns", type="str", help="Bipartite motif configuration: acceptable length of middle indeterminate component (comma-separated string of integers) [5,6]" )
	
	parser.add_option( "--bipart_second", type="str", help="Bipartite motif configuration: acceptable length of second determinate component (comma-separated string of integers) [3,4]" )

	parser.add_option( "--mod_bases", type="str", help="String containing bases to query for mods. Changing this is not recommended ['A']" )

	parser.add_option( "--minAcc", type="float", help="Min subread accuracy of read [0.8]" )

	parser.add_option( "--minMapQV", type="float", help="Min mapping QV of aligned read [240]" )

	parser.add_option( "--procs", type="int", help="Number of cores to use [4]" )

	parser.add_option( "--N_reads", type="int", help="Number of qualifying reads to include in analysis [1000000000]" )
	
	parser.add_option( "--min_motif_count", type="int", help="Number of motif sites required in WGA data to be included in controls dictionary [10]" )

	parser.add_option( "--control_pkl_name", type="str", help="Filename to save control IPD data from WGA sequencing [control_ipds.pkl]" )
	
	parser.set_defaults( logFile="log.buildcontrols",        \
						 info=False,                         \
						 debug=False,                        \
						 ref=None,                           \
						 subreadlength_min=100,              \
						 readlength_min=100,                 \
						 min_kmer=4,                         \
						 max_kmer=6,                         \
						 no_bipartite=False,                 \
						 bipart_first="3,4",                 \
						 bipart_Ns="5,6",                    \
						 bipart_second="3,4",                \
						 mod_bases="A",                      \
						 minAcc=0.8,                         \
						 minMapQV=240,                       \
						 procs=4,                            \
						 N_reads=1000000000,                 \
						 min_motif_count=10,                 \
						 control_pkl_name="control_ipds.pkl")

	opts, args         = parser.parse_args( )
	control_aln_fn     = __check_input( opts, args, parser )

	if opts.no_bipartite:
		opts.bipartite = False
	else:
		opts.bipartite = True

	############################################
	# Define the types of bipartite motifs to 
	# include in the analysis. This describes the
	# acceptable sizes of the three components of
	# bipartite motifs. For example, the motif 
	# ACCT/NNNNN/CTT (first/Ns/last) would be 
	# described by 4/5/3.
	
	first  = map(lambda x: int(x), opts.bipart_first.split(","))
	middle = map(lambda x: int(x), opts.bipart_Ns.split(","))
	second = map(lambda x: int(x), opts.bipart_second.split(","))
	opts.bipart_config   = [(first), (middle), (second)]
	
	# As set, acceptible bipartite motifs would have
	# the following component lengths.
	# First: 3 or 4 ACGT bases
	# Ns: 5 or 6 unspecified N bases
	# Last: 3 or 4 ACGT bases
	############################################

	opts.control_tmp     = "ctrl_tmp"
	opts.tmp             = "tmp"
	opts.minContigLength = 0
	opts.comp_kmer       = 5
	# opts.h5_type         = "cmp"
	opts.cross_cov_bins  = None
	opts.sam             = None
	opts.motifs_file     = None
	opts.skip_motifs     = None
	
	opts.control_pkl_name = os.path.abspath(opts.control_pkl_name)
	opts.ref              = os.path.abspath(opts.ref)

	return opts,control_aln_fn

def __initLog( opts ):
	"""Sets up logging based on command line arguments. Allows for three levels of logging:
	logging.error( ): always emitted
	logging.info( ) : emitted with --info or --debug
	logging.debug( ): only with --debug"""

	if os.path.exists(opts.logFile):
		os.remove(opts.logFile)

	logLevel = logging.DEBUG if opts.debug \
				else logging.INFO if opts.info \
				else logging.ERROR

	logger = logging.getLogger("")
	logger.setLevel(logLevel)
	
	# create file handler which logs even debug messages
	fh = logging.FileHandler(opts.logFile)
	fh.setLevel(logLevel)
	
	# create console handler with a higher log level
	ch = logging.StreamHandler()
	ch.setLevel(logLevel)
	
	# create formatter and add it to the handlers
	logFormat = "%(asctime)s [%(levelname)s] %(message)s"
	formatter = logging.Formatter(logFormat, "%Y-%m-%d %H:%M:%S")
	ch.setFormatter(formatter)
	fh.setFormatter(formatter)
	
	# add the handlers to logger
	logger.addHandler(ch)
	logger.addHandler(fh)

def __check_input( opts, args, parser ):
	control_aln_fn = os.path.abspath(args[0])

	if control_aln_fn[-7:]==".cmp.h5":
		opts.aln_ftype = "cmp"
		opts.aligned = True
	elif control_aln_fn[-4:]==".bam":
		opts.aln_ftype = "bam"
		opts.aligned = True
	else:
		parser.error("Could not recognize valid input (BAM or cmp.h5 file of aligned reads): %s" % control_aln_fn)

	if opts.aln_ftype=="bam" and opts.ref==None:
		parser.error("With BAM input, must specify reference fasta using --ref. Fasta must be indexed (use samtools faidx).")

	# if opts.contigs==None:
	# 	parser.error("Please specify the fasta file used for the alignments in %s!" % control_aln_fn)

	if len(args) != 1:
		parser.error( "Expected 1 argument." )

	return control_aln_fn

def process_contig_chunk( args ):
	chunk_id        = args[0]
	cut_CMDs        = args[1]
	kmers           = args[2]
	cols_chunk      = args[3]
	n_chunks        = args[4]
	min_motif_count = args[5]
	logging.info("  - Control data: chunk %s/%s" % ((chunk_id+1), (n_chunks+1)))
	control_means = {}
	
	for cut_CMD in cut_CMDs:
		sts,stdOutErr = mbin.run_OS_command( cut_CMD )
	
	fns                = map(lambda x: x.split("> ")[-1], cut_CMDs)
	control_ipds_sub   = np.loadtxt(fns[0], dtype="float")
	control_ipds_N_sub = np.loadtxt(fns[1], dtype="int")
	# If there is only one row (read) for this contig, still treat as
	# a 2d matrix of many reads
	control_ipds_sub   = np.atleast_2d(control_ipds_sub)
	control_ipds_N_sub = np.atleast_2d(control_ipds_N_sub)
	
	not_found     = 0
	for j in range(len(cols_chunk)):
		motif = kmers[cols_chunk[j]]
		if np.sum(control_ipds_N_sub[:,j])>=min_motif_count:
			if np.sum(control_ipds_N_sub[:,j])>0:
				control_mean = np.dot(control_ipds_sub[:,j], control_ipds_N_sub[:,j]) / np.sum(control_ipds_N_sub[:,j])
			else:
				control_mean = 0
			control_means[motif] = control_mean
		else:
			not_found += 1

	return control_means,not_found

class ControlRunner:
	def __init__( self, wga_h5, opts ):
		"""
		Point to the appropriate WGA sequencing data files
		to generate the control IPD values for the motifs. 
		"""
		self.control_aln_fn = wga_h5
		self.opts       = opts
		self.orig_dir   = os.getcwd()

	def build_insilico_controls( self ):
		"""
		To be added...
		"""
		pass

	def goto_control_output_dir( self ):
		"""
		Create directory where data from control reads 
		will be gathered and analyzed.
		"""
		# Make control directory
		if os.path.exists(self.opts.control_tmp):
			shutil.rmtree(self.opts.control_tmp)
		os.mkdir(self.opts.control_tmp)
		os.chdir(self.opts.control_tmp)
		
		# Make tmp directory inside control directory
		if os.path.exists(self.opts.tmp):
			shutil.rmtree(self.opts.tmp)
		os.mkdir(self.opts.tmp)

	def return_to_orig_dir( self ):
		"""
		Back out of the control directory.
		"""
		os.chdir(self.orig_dir)

	def scan_WGA_aligns( self ):
		"""
		Get some necessary information about the WGA cmp.h5 
		being used to generate the control IPD data.
		"""
		self.opts.aln_fn_labels                           = {}
		self.opts.aln_fn_contig_lens                      = {}
		self.opts.aln_fn_labels[self.control_aln_fn]      = "control"
		self.opts.aln_fn_contig_lens[self.control_aln_fn] = {}
		
		# reader = CmpH5Reader(self.control_aln_fn)
		reader = openIndexedAlignmentFile(self.control_aln_fn)
		for entry in reader.referenceInfoTable:
			name      = entry[3]
			length    = entry[4]
			slug_name = mbin.slugify(name)
			self.opts.aln_fn_contig_lens[self.control_aln_fn][slug_name] = length
		reader.close()

		return self.opts

	def analyze_WGA_reads( self ):
		"""
		Launch read scanning pipeline for building up
		control IPD values for motifs.
		"""
		control_fns = glob.glob( os.path.join(self.opts.tmp, "*.tmp"))
		ftypes      = set( map(lambda x: "_".join(os.path.basename(x).split("_")[2:]), control_fns) )
		for ftype in ftypes:
			if ftype in ["compkmers.tmp", "ipdskmers.tmp"]:
				# first_fn = glob.glob( os.path.join(self.opts.tmp, "unitig_*_%s" % ftype) )[0]
				first_fn = glob.glob( os.path.join(self.opts.tmp, "*_%s" % ftype) )[0]
				shutil.copy( first_fn, os.path.join(self.opts.tmp, "control_%s" % ftype) )
				
				ftype_fns = glob.glob( os.path.join(self.opts.tmp, "*_%s" % ftype))
				to_rm     = [fn for fn in ftype_fns if not os.path.basename(fn).startswith("control_")]
				for fn in to_rm:
					os.remove(fn)
			else:
				ftype_fns = glob.glob( os.path.join(self.opts.tmp, "*_%s" % ftype) )
				to_cat    = [fn for fn in ftype_fns if not os.path.basename(fn).startswith("control_")]
				to_cat.sort()
				outname = os.path.join(self.opts.tmp, "control_%s" % ftype )
				mbin.cat_list_of_files(to_cat, outname)
		
	def chunk_control_matrices( self, control_ipds_fn, control_ipds_N_fn, control_kmers_fn ):
		"""

		"""
		kmers       = np.atleast_1d(np.loadtxt(control_kmers_fn, dtype="str"))
		fns         = [control_ipds_fn, control_ipds_N_fn]
		n_chunks    = 99
		chunksize   = int(math.ceil(float( len(kmers)/n_chunks )))
		cols_chunks = list(chunks( range(len(kmers)), chunksize ))
		args        = []
		for i,cols_chunk in enumerate(cols_chunks):
			cut_CMDs = []
			for fn in fns:
				cut_cols = "%s-%s" % ((cols_chunk[0]+1), (cols_chunk[-1]+1))
				in_fn    = fn
				out_fn   = fn+".sub.%s" % i
				cut_CMD  = "cut -d$\'\\t\' -f%s %s > %s" % (cut_cols, in_fn, out_fn)
				cut_CMDs.append(cut_CMD)
			args.append( (i, cut_CMDs, kmers, cols_chunk, n_chunks, self.opts.min_motif_count) )
		
		results = mbin.launch_pool(self.opts.procs, process_contig_chunk, args)
		
		logging.info("Combining motifs from all chunks of control data...")
		not_found     = 0
		control_means = {}
		for i,result in enumerate(results):
			not_found += result[1]
			for motif in result[0].keys():
				control_means[motif] = result[0][motif]
		logging.info("Done.")

		return control_means,not_found

	def combine_control_data_from_contigs( self ):
		"""
		If control WGA data contains multiple contigs, this 
		will combine them into one file for each data type
		so that the control IPD dictionary will be generated
		using data from all contigs.
		"""
		contigs_fns    = glob.glob( os.path.join(self.opts.tmp, "control_*.tmp") )
		labels_fns     = []
		strands_fns    = []
		lengths_fns    = []
		readnames_fns  = []
		ipds_fns       = []
		ipds_N_fns     = []
		comp_N_fns     = []
		comp_kmers_fns = []
		ipds_kmers_fns = []
		for fn in contigs_fns:
			if   fn.find("_labels.")>-1:
				labels_fns.append(fn)
			elif fn.find("_strand.")>-1:
				strands_fns.append(fn)
			elif fn.find("_lengths.")>-1:
				lengths_fns.append(fn)
			elif fn.find("_readnames.")>-1:
				readnames_fns.append(fn)
			elif fn.find("_ipds.")>-1:
				ipds_fns.append(fn)
			elif fn.find("_ipdsN.")>-1:
				ipds_N_fns.append(fn)
			elif fn.find("_compN.")>-1:
				comp_N_fns.append(fn)
			elif fn.find("_compkmers.")>-1:
				comp_kmers_fns.append(fn)
			elif fn.find("_ipdskmers.")>-1:
				ipds_kmers_fns.append(fn)

		labels_fns.sort()
		strands_fns.sort()
		lengths_fns.sort()
		readnames_fns.sort()
		ipds_fns.sort()
		ipds_N_fns.sort()
		comp_N_fns.sort()

		mbin.cat_list_of_files(labels_fns,    "control_labels.tmp")
		mbin.cat_list_of_files(strands_fns,   "control_strands.tmp")
		mbin.cat_list_of_files(lengths_fns,   "control_lengths.tmp")
		mbin.cat_list_of_files(readnames_fns, "control_names.tmp")
		mbin.cat_list_of_files(ipds_fns,      "control_ipds.tmp")
		mbin.cat_list_of_files(ipds_N_fns,    "control_ipdsN.tmp")
		mbin.cat_list_of_files(comp_N_fns,    "control_compN.tmp")
		
		shutil.copy(comp_kmers_fns[0], "control_compkmers.tmp")
		shutil.copy(ipds_kmers_fns[0], "control_ipdskmers.tmp")
		x = [os.remove(fn) for fn in comp_kmers_fns]
		x = [os.remove(fn) for fn in ipds_kmers_fns]

	def build_control_IPD_dict( self, motifs, bi_motifs ):
		"""

		"""
		control_ipds_fn   = glob.glob( "control_ipds.tmp" )
		control_ipds_N_fn = glob.glob( "control_ipdsN.tmp")
		control_kmers_fn  = glob.glob( "control_ipdskmers.tmp")

		if (len(control_ipds_fn)>1 or len(control_ipds_N_fn)>1 or len(control_kmers_fn)>1):
			raise Exception("*** Double check the control files. There should not be multiples for a file type.")

		control_means,not_found = self.chunk_control_matrices(control_ipds_fn[0], control_ipds_N_fn[0], control_kmers_fn[0])

		if not_found > 0:
			logging.info("")
			logging.warning("WARNING: could not find sufficient instances (>=%s) for %s motifs (out of %s total) in control data!" % (self.opts.min_motif_count, not_found, (len(motifs)+len(bi_motifs))))
			logging.warning("   * If this is alarming, try reducing --min_motif_count or increasing --N_reads, although you just might not have those motifs in your reference sequence.")
		
		logging.info("")
		logging.info("Writing control data to a pickled file: %s" % self.opts.control_pkl_name)
		pickle.dump( control_means, open( self.opts.control_pkl_name, "wb" ) )

		return control_means

if __name__ == "__main__":
	main()


import os,sys,glob
import re
import shutil
import math
import multiprocessing
from pbcore.io.align.CmpH5IO import CmpH5Reader
from pbcore.io import openIndexedAlignmentFile
import subprocess
import numpy as np
import pysam
from collections import OrderedDict,Counter,defaultdict
import logging
import mbin
import read_scanner
from operator import itemgetter
import pickle
import unicodedata

import warnings
warnings.simplefilter("error")
warnings.filterwarnings("ignore",category=DeprecationWarning)


class subread_motif_processor:
	def __init__( self, align_fn, chunk_id, idx, N_target_reads, motifs, bi_motifs, opts ):
		self.align_fn       = align_fn
		self.chunk_id       = chunk_id
		self.idx            = idx
		self.N_target_reads = N_target_reads
		self.motifs         = motifs
		self.bi_motifs      = bi_motifs
		self.opts           = opts

	def __call__( self ):
		
		class ipd_entry:
			def __init__( self, tup ):
				"""
				"""
				self.ref_base  = tup[0]
				self.ipd       = tup[1]
				self.ref_pos   = tup[2]

		class subread:
			def __init__( self, align_fn, alignment, label, opts ):
				leftAnchor   = 1
				rightAnchor  = 1
				self.entries = {}
				self.opts    = opts

				self.subname    = alignment.readName
				alignedLength   = alignment.referenceSpan
				self.refName    = alignment.referenceInfo[3]
				zmw             = alignment.HoleNumber


				###########################################
				# self.mol        = alignment.MoleculeID
				self.mol        = alignment.HoleNumber
				###########################################
				

				if alignment.isForwardStrand:
					self.strand = 0
				else:
					self.strand = 1

				self.ref_bases  = alignment.reference()

				read_calls      = alignment.transcript()
				ref_pos         = list(alignment.referencePositions())
				IPD             = list(alignment.IPD())
				self.label      = self.opts.aln_fn_labels[align_fn]

				error_mk = []
				for read_call in read_calls:
					# Go through all entries and flag which positions are MM/indels
					if read_call != "M":
						# Mismatch or indel at this position!
						error_mk.append(1)
					else:
						error_mk.append(0)
				
				# Get the indices of all the non-matches
				error_idx = [i for (i,val) in enumerate(error_mk) if val == 1]
				for error_id in error_idx:
					try:
						for j in range(leftAnchor):
							error_mk[error_id - (j+1)] = 1
						for j in range(rightAnchor):
							error_mk[error_id + (j+1)] = 1
					except IndexError:
						pass
				error_mk = np.array(error_mk)

				ipds     = np.array(IPD) / self.opts.fps
				
				strands  = np.array([self.strand] * len(read_calls))

				self.ref_bases  = np.array(list(self.ref_bases))
				self.ref_pos    = np.array(ref_pos)
				read_calls      = np.array(list(read_calls))

				# Mark the error positions, but leave them in the sequence so
				# we can pull out intact motifs from contiguous correct bases
				self.ref_bases[error_mk==1]  = "*"
				read_calls[error_mk==1]      = "*"
				ipds[error_mk==1]            = -9
				strands[error_mk==1]         = -9

				# Attach these IPD entries to the subread object
				for i,tup in enumerate(zip(self.ref_bases, ipds, self.ref_pos)):
					entry = ipd_entry(tup)
					self.entries[ self.ref_pos[i] ] = ipd_entry(tup)

				# self.cap_outliers()

				self.subread_normalize()

			def cap_outliers( self, max_ipd=10 ):
				"""
				Cap the outlier IPDs at max_ipd seconds.
				"""
				for read_pos,entry in self.entries.iteritems():
					entry.ipd = min(entry.ipd, max_ipd)

			def subread_normalize( self ):
				"""
				Every IPD entry needs to be normalized by the mean IPD of its subread.
				"""
				if len(self.entries) == 0:
					# Nothing to do here.
					return self.entries

				# First populate list of all IPDs per subread. Will use to get normalization factor.
				subread_vals = []
				for entry in self.entries.values():
					# Only do if this IPD is NOT from an error position
					if entry.ipd != -9:
						subread_vals.append(entry.ipd)

				rawIPDs = np.array(map(lambda x: math.log(x + 0.001), subread_vals))
				nfs     = rawIPDs.mean()

				for pos, entry in self.entries.iteritems():
					if entry.ipd == -9:
						newIPD = -9
					else:
						newIPD = math.log(entry.ipd + 0.001) - nfs
					
					entry.ipd = newIPD

			def zip_bases_and_IPDs( self ):
				"""
				Reassemble the read and IPD values using the subread normalized IPDs
				"""
				od        = OrderedDict(sorted(self.entries.items()))
				ref       = []
				ref_pos   = []
				self.ipds = []
				for read_pos, entry in od.items():
					ref.append(entry.ref_base)
					ref_pos.append(entry.ref_pos)
					self.ipds.append(entry.ipd)
				self.ref_str  = "".join(ref)
				self.ref_pos  = ref_pos

		if self.opts.aln_ftype=="cmp":
			reader = CmpH5Reader(self.align_fn)
		elif self.opts.aln_ftype=="bam":
			reader = openIndexedAlignmentFile(self.align_fn, self.opts.ref)
		else:
			raise Exception("Unrecognized alignment filetype (must be *.cmp.h5 or *.bam): %s" % self.align_fn)

		def get_fps(align_fn):
			"""
			For *.cmp.h5 files, frame rate (fps) is include in each alignment.
			For *.bam files, the frame rate is encoded in the file header (FRAMERATEHZ)
			"""
			if self.opts.aln_ftype=="cmp":
				# Read frame rate directly from a cmp.h5 alignment
				reader    = CmpH5Reader(align_fn)
				alignment = reader[0]
				fps       = alignment.movieInfo[2]
			
			elif self.opts.aln_ftype=="bam":
				# Isolate description (DS) from read group (RG) in BAM header
				bam     = pysam.AlignmentFile(align_fn, "rb")
				h       = bam.header
				rg_ds_l = h.as_dict()["RG"][0]["DS"].split(";")
				rg_ds_d = dict([ (x.split("=")[0], x.split("=")[1]) for x in rg_ds_l])
				fps     = float(rg_ds_d["FRAMERATEHZ"])

			return fps

		# Pull the frame rate value from the alignment file
		self.opts.fps = get_fps(self.align_fn)

		read_refs     = {}
		read_SMp      = {}
		read_SMp_N    = {}
		read_comps    = {}
		read_labs     = {}
		contig_SCp    = {}
		i             = 0
		n_mols        = 0

		cwd = os.getcwd()

		# Periodically (after <chunksize> alignments) write out data to a contig-specific tmp file
		chunksize     = 10
		self.chunkdir = "chunk_%s" % self.chunk_id
		if os.path.exists(os.path.join(self.opts.tmp, self.chunkdir)):
			shutil.rmtree(os.path.join(self.opts.tmp, self.chunkdir))
		os.mkdir(os.path.join(self.opts.tmp, self.chunkdir))
		to_dump = defaultdict(list)

		def dump_data_to_contig_files( refName, to_dump, read_labs ):
			refName           = mbin.slugify(refName)
			ref_subname_fn    = "%s_readnames.tmp"    % refName
			ref_label_fn      = "%s_labels.tmp"       % refName
			ref_length_fn     = "%s_lengths.tmp"      % refName
			ref_ipds_fn       = "%s_ipds.tmp"         % refName
			ref_ipds_N_fn     = "%s_ipdsN.tmp"        % refName
			ref_comp_N_fn     = "%s_compN.tmp"        % refName
			ref_strand_fn     = "%s_strand.tmp"       % refName

			self.tmp_fns.add( os.path.join(self.chunkdir, ref_subname_fn) )
			self.tmp_fns.add( os.path.join(self.chunkdir, ref_label_fn) )
			self.tmp_fns.add( os.path.join(self.chunkdir, ref_length_fn) )
			self.tmp_fns.add( os.path.join(self.chunkdir, ref_ipds_fn) )
			self.tmp_fns.add( os.path.join(self.chunkdir, ref_ipds_N_fn) )
			self.tmp_fns.add( os.path.join(self.chunkdir, ref_comp_N_fn) )
			self.tmp_fns.add( os.path.join(self.chunkdir, ref_strand_fn) )
			f_subnames   = open(os.path.join(self.opts.tmp, self.chunkdir, ref_subname_fn),  "a")
			f_labels     = open(os.path.join(self.opts.tmp, self.chunkdir, ref_label_fn),    "a")
			f_lengths    = open(os.path.join(self.opts.tmp, self.chunkdir, ref_length_fn),   "a")
			f_ipds       = open(os.path.join(self.opts.tmp, self.chunkdir, ref_ipds_fn),     "a")
			f_ipds_N     = open(os.path.join(self.opts.tmp, self.chunkdir, ref_ipds_N_fn),   "a")
			f_comp_N     = open(os.path.join(self.opts.tmp, self.chunkdir, ref_comp_N_fn),   "a")
			f_strand     = open(os.path.join(self.opts.tmp, self.chunkdir, ref_strand_fn),   "a")
			self.tmp_fs.add(f_subnames)
			self.tmp_fs.add(f_labels)
			self.tmp_fs.add(f_ipds)
			self.tmp_fs.add(f_ipds_N)
			self.tmp_fs.add(f_comp_N)
			self.tmp_fs.add(f_strand)
			
			if self.opts.motifs_file!=None and self.opts.subtract_control:
				control_ipds_d = pickle.load( open(self.opts.control_pkl_name,"rb" ) )

			for i,(subread_ipds,subread_comps,readname,subread_length,strand) in enumerate(to_dump[refName]):
				ipd_kmers   = [motif                  for motif in subread_ipds.iterkeys()]
				ipd_means   = [subread_ipds[motif][1] for motif in subread_ipds.iterkeys()]
				ipd_counts  = [subread_ipds[motif][0] for motif in subread_ipds.iterkeys()]

				ipd_means = []
				if self.opts.motifs_file!=None and self.opts.subtract_control:
					for motif in subread_ipds.iterkeys():
						if subread_ipds[motif][1] != 0.0:
							w_control_sub = subread_ipds[motif][1] - control_ipds_d[motif]
							ipd_means.append(w_control_sub)
						else: # Don't subtract control if no ipd values are available (i.e. IPD score == 0.0)
							ipd_means.append(subread_ipds[motif][1])
				else:
					for motif in subread_ipds.iterkeys():
						ipd_means.append(subread_ipds[motif][1])

				comp_kmers  = np.array( [motif   for motif,ipds in subread_comps.items()] )
				comp_counts = np.array( [ipds    for motif,ipds in subread_comps.items()] )
				if i==0 and refName not in self.refName_has_header:
					ref_ipds_kmers_fn = "%s_ipdskmers.tmp" % refName
					ref_comp_kmers_fn = "%s_compkmers.tmp" % refName
					f_ipds_kmers      = open(os.path.join(self.opts.tmp, self.chunkdir, ref_ipds_kmers_fn), "a")
					f_comp_kmers      = open(os.path.join(self.opts.tmp, self.chunkdir, ref_comp_kmers_fn), "a")
					ipds_kmers_str    = "\t".join(ipd_kmers)
					comp_kmers_str    = "\t".join(comp_kmers)
					f_ipds_kmers.write("%s\n" % ipds_kmers_str)
					f_comp_kmers.write("%s\n" % comp_kmers_str)
					f_ipds_kmers.close()
					f_comp_kmers.close()
					self.refName_has_header.add(refName)
				ipds_str        = "\t".join(map(lambda x: str(round(x,3)), ipd_means))
				ipds_N_str      = "\t".join(map(lambda x: str(x),          ipd_counts))
				comp_counts_str = "\t".join(map(lambda x: str(x),          comp_counts))
				f_subnames.write(  "%s\n" % readname)
				f_labels.write(    "%s\n" % read_labs[readname])
				f_lengths.write(   "%s\n" % subread_length)
				f_ipds.write(      "%s\n" % ipds_str)
				f_ipds_N.write(    "%s\n" % ipds_N_str)
				f_comp_N.write(    "%s\n" % comp_counts_str)
				f_strand.write(    "%s\n" % strand)

			for f in self.tmp_fs:
				f.close()
		self.tmp_fs             = set()
		self.tmp_fns            = set()
		self.refName_has_header = set()
		to_check = reader[self.idx]
		for alignment in to_check:
			ref_contig = mbin.slugify(alignment.referenceInfo[3])
			label      = self.opts.aln_fn_labels[self.align_fn]
			ref_len    = self.opts.aln_fn_contig_lens[self.align_fn][ref_contig]
			if ref_len >= self.opts.minContigLength and alignment.referenceSpan >= self.opts.readlength_min and alignment.MapQV >= self.opts.minMapQV:
				to_get    = min(self.N_target_reads, len(self.idx))
				incr      = to_get/10
				readname  = "/".join(alignment.readName.split("/")[:-1])
				if len(read_labs.keys())%incr==0 and not read_labs.get(readname):
					logging.info("...chunk %s\t- mol %s/%s (%.1f%%)" % (self.chunk_id, n_mols, to_get, 100*n_mols/to_get))

				read_labs[readname] = label
				read_refs[readname] = ref_contig

				sub = subread( self.align_fn, alignment, label, self.opts )
				sub.zip_bases_and_IPDs()
				subread_ipds,subread_comps = read_scanner.scan_motifs( "aligned",      \
																	   sub.ipds,       \
																	   sub.ref_str,    \
																	   sub.strand,     \
																	   self.motifs,    \
																	   self.bi_motifs, \
																	   self.opts )
				
				to_dump[ref_contig].append( (subread_ipds,subread_comps,readname,len(sub.ref_str),sub.strand) )
				# Dump subread IPD and comp data to contig-specific file
				if len(to_dump[ref_contig])%chunksize==0 and len(to_dump[ref_contig])!=0:
					dump_data_to_contig_files( ref_contig, to_dump, read_labs )
					to_dump[ref_contig] = []

				n_mols = len(read_labs.keys())
				i += 1
					
				if n_mols==self.N_target_reads:
					break

		for ref_contig in to_dump.keys():
			dump_data_to_contig_files( ref_contig, to_dump, read_labs )
		for f in self.tmp_fs:
			f.close()
		to_dump = defaultdict(list)

		if i==0:
			logging.info("Chunk %s: no qualifying reads found!" % self.chunk_id)
		
		logging.info("Chunk %s: found %s alignments (%s molecules) > %sbp in %s" % (self.chunk_id, i, len(read_labs.keys()), self.opts.readlength_min, os.path.basename(self.align_fn)))
		reader.close()

		return self.tmp_fns


import os,sys
from collections import Counter,defaultdict
from itertools import groupby
import math
import numpy as np
from pbcore.io.align.CmpH5IO import CmpH5Reader
from pbcore.io import openIndexedAlignmentFile
from pbcore.io.BasH5IO import BasH5Reader
import shutil
import re
import copy
import random
from scipy import stats
import multiproc
import read_scanner
import cmph5_read
import baxh5_read
import logging
import multiprocessing
import operator
import subprocess
import glob
from itertools import izip
import pickle
import pysam
import motif_tools
import unicodedata

def run_OS_command( CMD ):
	p         = subprocess.Popen(CMD, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
	stdOutErr = p.communicate()
	sts       = p.returncode
	if sts != 0:
		logging.warning("Failed command: %s" % CMD)
	return sts, stdOutErr

def launch_pool( procs, funct, args ):
	p    = multiprocessing.Pool(processes=procs)
	try:
		results = p.map(funct, args)
		p.close()
		p.join()
	except KeyboardInterrupt:
		p.terminate()
	return results

def cat_list_of_files( in_fns, out_fn, del_ins=True ):
	"""
	Given a list of filenames, cat them together into a single file. Then cleans up pre-catted
	single files.
	"""
	if len(in_fns)==0:
		raise Exception("There are no files to cat!")
		
	cat_CMD   = "cat %s > %s" % (" ".join(in_fns), out_fn)
	p         = subprocess.Popen(cat_CMD, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
	stdOutErr = p.communicate()
	sts       = p.returncode
	if sts != 0:
		raise Exception("Failed cat command: %s" % cat_CMD)
	if del_ins:
		for fn in in_fns:
			try:
				os.remove(fn)
			except OSError:
				pass
	return out_fn

def slugify(text):
	"""Generates an ASCII-only slug."""
	slug = unicodedata.normalize("NFKD",unicode(text)).encode("ascii", "ignore")
	slug = re.sub(r"[^\w]+", " ", slug)
	slug = "-".join(slug.lower().strip().split())
	return slug

def cat_contig_files_from_subprocs( tup ):
	contig    = tup[0]
	tmp       = tup[1]
	j         = tup[2]
	N_contigs = tup[3]
	logging.info("...contig %s (%s/%s)" % (contig, j, N_contigs))
	contig_fns = glob.glob( os.path.join(tmp, "chunk_*", "%s_*.tmp" % contig) )
	ftypes = set( map(lambda x: os.path.basename(x).split("_")[-1], contig_fns) )
	for ftype in ftypes:
		if ftype in ["compkmers.tmp", "ipdskmers.tmp"]:
			first_fn = glob.glob( os.path.join(tmp, "chunk_*", "%s_%s" % (contig, ftype)) )[0]
			shutil.copy( first_fn, os.path.join(tmp, "%s_%s" % (contig,ftype)))
		else:
			to_cat  = glob.glob( os.path.join(tmp, "chunk_*", "%s_%s" % (contig, ftype)) )
			to_cat.sort()
			outname = os.path.join(tmp, "%s_%s" % (contig,ftype) )
			cat_list_of_files(to_cat, outname, del_ins=False)

def cat_subreads_files_from_subprocs( subproc_tmp_files, movie_i, opts ):
	for ftype in subproc_tmp_files.keys():
		if ftype in ["compkmers.tmp", "ipdskmers.tmp"]:
			first_fn = glob.glob( os.path.join(opts.tmp, "chunk_*", "subreads_%s" % ftype) )[0]
			shutil.copy( first_fn, os.path.join(opts.tmp, "subreads_%s" % ftype.replace(".tmp", ".%s.tmp" % movie_i)))
		else:
			ftype_fns = list(subproc_tmp_files[ftype])
			sorted(ftype_fns)
			catted_name = os.path.join(opts.tmp, "subreads_%s" % ftype.replace(".tmp", ".%s.tmp" % movie_i))
			cat_list_of_files( ftype_fns, catted_name, del_ins=False )

def check_input_ftype(opts, args, parser):
	"""
	Make sure the input is in the form of either:
	  (a) a cmp.h5 file of aligned reads
	  (b) a bam file of aligned reads
	  (c) an unaligned bas.h5 file
	  (d) a FOFN of unaligned bas.h5 files

	Also make sure a fasta file of the assembled contigs is 
	specified if input is a *.cmp.h5 or *.bam file containing
	read alignments to those contigs.
	"""

	input_fn           = os.path.abspath(args[0])
	input_files        = []
	opts.aln_fn_labels = {}

	if input_fn[-7:]==".cmp.h5":
		print "Found cmp.h5 of aligned reads:"
		opts.input_ftype                  = "cmp"
		opts.aln_fn_contig_lens           = {}
		opts.aln_fn_contig_lens[input_fn] = {}

		input_files.append(input_fn)
		print "  -- %s" % input_fn
		print "Getting contig information from %s..." % input_fn
		reader = CmpH5Reader(input_fn)
		for entry in reader.referenceInfoTable:
			name                                         = entry[3]
			length                                       = entry[4]
			slug_name                                    = slugify(name)
			opts.aln_fn_contig_lens[input_fn][slug_name] = length
			opts.aln_fn_labels[input_fn]                 = "ignore"
		reader.close()
		opts.aligned   = True
		opts.aln_ftype = "cmp"

	elif input_fn[-4:]==".bam":
		print "Found BAM file of aligned reads:"
		opts.input_ftype                  = "bam"
		opts.aln_fn_contig_lens           = {}
		opts.aln_fn_contig_lens[input_fn] = {}

		input_files.append(input_fn)
		print "  -- %s" % input_fn 
		print "Getting contig information from %s..." % input_fn
		if  opts.contigs==None:
			parser.error("With aligned (BAM or cmp.h5) input, must specify reference fasta using --contigs. For aligned BAM input, fasta must be indexed (use samtools faidx).")
		else:
			# reader = openIndexedAlignmentFile(input_fn, opts.contigs)
			reader = openIndexedAlignmentFile(input_fn)
		for entry in reader.referenceInfoTable:
			name                                         = entry[3]
			length                                       = entry[4]
			slug_name                                    = slugify(name)
			opts.aln_fn_contig_lens[input_fn][slug_name] = length
			opts.aln_fn_labels[input_fn]                 = "ignore"
		reader.close()
		opts.aligned   = True
		opts.aln_ftype = "bam"

	elif input_fn[-7:]==".bas.h5":
		print "Found bas.h5 of unaligned reads:"
		opts.input_ftype = "bas"
		input_files.append(input_fn)
		opts.aln_fn_labels[input_fn] = "ignore"
		print "  -- %s" % input_fn
		opts.aligned = False

	elif input_fn[-5:]==".fofn":
		print "Found FOFN of bas.h5 files:"
		opts.input_ftype = "bas"
		fns              = map(lambda x: x.strip("\n"), np.atleast_1d(open(input_fn, "r").read()))
		input_files      = fns
		for fn in fns:
			if fn[-7:]!=".bas.h5":
				parser.error("Expected to find a *.bas.h5 file, instead found %s" % fn)
			else:
				print "  -- %s" % fn
				opts.aln_fn_labels[fn] = "ignore"
		opts.aligned = False

	return input_files, opts

class mbinRunner:
	def __init__( self, opts ):
		"""
		"""
		self.opts = opts
		self.fns  = {}
	
		##########################################################
		# Define output filenames
		##########################################################
		if self.opts.aligned:
			#############################################
			# Define the alignment-level output filenames
			#############################################
			self.fns["read_names"]           = "reads.names"
			self.fns["read_refs"]            = "reads.refs"
			self.fns["read_labels"]          = "reads.labels"
			self.fns["read_lengths"]         = "reads.lengths"
			self.fns["read_strands"]         = "reads.strands"
			self.fns["read_comp_kmers"]      = "reads.comp_kmers"
			self.fns["read_comp_counts"]     = "reads.comp_counts"
			self.fns["read_comp"]            = "reads.comp"
			self.fns["read_comp_2D"]         = "reads.comp.2D"
			self.fns["read_comp_2D_z"]       = "reads.comp.2D.zscores"
			self.fns["read_SMp_kmers"]       = "reads.SMp_kmers"
			self.fns["read_SMp_counts"]      = "reads.SMp_counts"
			self.fns["read_SMp"]             = "reads.SMp"
			self.fns["read_SMp_2D"]          = "reads.SMp.2D"
			self.fns["read_SMp_2D_z"]        = "reads.SMp.2D.zscores"
			self.fns["read_4D_z"]            = "reads.combined.4D.zscores"
			self.fns["read_combo_2D"]        = "reads.combined.2D"

			# #############################################
			# # Define the contig-level output filenames
			# #############################################
			self.fns["contig_names"]         = "contigs.names"
			self.fns["contig_labels"]        = "contigs.labels"
			self.fns["contig_lengths"]       = "contigs.lengths"
			self.fns["contig_comp"]          = "contigs.comp"
			self.fns["contig_comp_2D"]       = "contigs.comp.2D"
			self.fns["contig_comp_2D_z"]     = "contigs.comp.2D.zscores"
			self.fns["contig_cov_comp_3D"]   = "contigs.cov_comp.3D"
			self.fns["contig_cov_comp_2D"]   = "contigs.cov_comp.2D"
			self.fns["contig_cov_comp_2D_z"] = "contigs.cov_comp.2D.zscores"
			self.fns["contig_SCp"]           = "contigs.SCp"
			self.fns["contig_SCp_N"]         = "contigs.SCp_N"
			self.fns["contig_SCp_2D"]        = "contigs.SCp.2D"
			self.fns["contig_SCp_2D_z"]      = "contigs.SCp.2D.zscores"
			self.fns["contig_cov"]           = "contigs.cov"
			self.fns["contig_4D_z"]          = "contigs.combined.4D.zscores"
			self.fns["contig_combo_2D"]      = "contigs.combined.2D"
		
			if self.opts.cross_cov_bins!=None:
				self.fns["bin_names"]        = "bins.names"
				self.fns["bin_sizes"]        = "bins.sizes"
				self.fns["bin_SCp"]          = "bins.SCp"
				self.fns["bin_SCp_N"]        = "bins.SCp_N"

		else:
			#############################################
			# Define the unaligned read-level output filenames
			#############################################
			self.fns["read_names"]           = "reads.names"
			self.fns["read_labels"]          = "reads.labels"
			self.fns["read_lengths"]         = "reads.lengths"
			self.fns["read_comp_kmers"]      = "reads.comp_kmers"
			self.fns["read_comp_counts"]     = "reads.comp_counts"
			self.fns["read_comp"]            = "reads.comp"
			self.fns["read_comp_2D"]         = "reads.comp.2D"
			self.fns["read_comp_2D_z"]       = "reads.comp.2D.zscores"
			self.fns["read_SMp_kmers"]       = "reads.SMp_kmers"
			self.fns["read_SMp_counts"]      = "reads.SMp_counts"
			self.fns["read_SMp"]             = "reads.SMp"
			self.fns["read_SMp_2D"]          = "reads.SMp.2D"
			self.fns["read_SMp_2D_z"]        = "reads.SMp.2D.zscores"
			self.fns["read_4D_z"]            = "reads.combined.4D.zscores"
			self.fns["read_combo_2D"]        = "reads.combined.2D"
			
			if self.opts.sam!=None:
				####################################################
				# Define contig filenames when SAM file used to map reads --> contigs
				####################################################
				self.fns["read_refs"]            = "reads.ccs.refs"
				self.fns["contig_names"]         = "contigs.names"
				self.fns["contig_labels"]        = "contigs.labels"
				self.fns["contig_lengths"]       = "contigs.lengths"
				self.fns["contig_comp"]          = "contigs.comp"
				self.fns["contig_comp_2D"]       = "contigs.comp.2D"
				self.fns["contig_comp_2D_z"]     = "contigs.comp.2D.zscores"
				self.fns["contig_cov_comp_3D"]   = "contigs.cov_comp.3D"
				self.fns["contig_cov_comp_2D"]   = "contigs.cov_comp.2D"
				self.fns["contig_cov_comp_2D_z"] = "contigs.cov_comp.2D.zscores"
				self.fns["contig_SCp"]           = "contigs.SCp"
				self.fns["contig_SCp_N"]         = "contigs.SCp_N"
				self.fns["contig_SCp_2D"]        = "contigs.SCp.2D"
				self.fns["contig_SCp_2D_z"]      = "contigs.SCp.2D.zscores"
				self.fns["contig_cov"]           = "contigs.cov"
				self.fns["contig_4D_z"]          = "contigs.combined.4D.zscores"
				self.fns["contig_combo_2D"]      = "contigs.combined.2D"

	def chunks( self, l, n ):
		"""
		Yield successive n-sized chunks from l.
		"""
		for i in xrange(0, len(l), n):
			yield l[i:i+n]

	def launch_data_loader( self, h5_file, N_reads, movie_i, opts ):
		logging.info("Loading data from %s..." % h5_file)
		results = self.launch_subprocs( h5_file, N_reads, opts )
		if opts.aligned:
			contig_tmps = defaultdict(list)
			for p,result in enumerate(results):
				for fn in result:
					contig = "_".join(fn.split("/")[1].split("_")[:-1])
					contig_tmps[contig].append(fn)
			logging.info("Catting contig-specific data from parallel processes...")
			args    = [(contig, opts.tmp, i, len(contig_tmps.keys())) for i,contig in enumerate(contig_tmps.keys())]
			results = launch_pool( opts.procs, cat_contig_files_from_subprocs, args )
		else:
			logging.info("Catting subprocess-specific data from parallel processes...")
			subproc_tmp_files = defaultdict(list)
			for result in results:
				for fn in result:
					ftype = fn.split(os.sep)[-1].split("subreads_")[-1]
					subproc_tmp_files[ftype].append(os.path.join(opts.tmp, fn))
			
			results = cat_subreads_files_from_subprocs( subproc_tmp_files, movie_i, opts )			
			logging.info("Done.")	

		for chunkdir in glob.glob( os.path.join(opts.tmp, "chunk_*")):
			shutil.rmtree(chunkdir)

	def fasta_iter( self, fasta_name ):
		"""
		Given a fasta file, yield tuples of (header, sequence).
		"""
		fh = open(fasta_name)
		# ditch the boolean (x[0]) and just keep the header or sequence since
		# we know they alternate.
		faiter = (x[1] for x in groupby(fh, lambda line: line[0] == ">"))
		for header in faiter:
			# drop the ">"
			header = header.next()[1:].strip()
			# join all sequence lines to one.
			seq = "".join(s.strip() for s in faiter.next())
			yield header, seq

	def combine_read_level_barcodes_across_contigs( self ):
		contigs_fns = glob.glob( os.path.join(self.opts.tmp, "*_read_*.tmp") )
		labels_fns     = []
		lengths_fns    = []
		refs_fns       = []
		readnames_fns  = []
		strands_fns    = []
		ipds_fns       = []
		ipds_N_fns     = []
		comp_N_fns     = []
		comp_kmers_fns = []
		ipds_kmers_fns = []
		for fn in contigs_fns:
			if   fn.find("_labels.tmp")>-1:
				labels_fns.append(fn)
			elif fn.find("_lengths.tmp")>-1:
				lengths_fns.append(fn)
			elif fn.find("_contig.tmp")>-1:
				refs_fns.append(fn)
			elif fn.find("_names.tmp")>-1:
				readnames_fns.append(fn)
			elif fn.find("_strands.tmp")>-1:
				strands_fns.append(fn)
			elif fn.find("_ipds.tmp")>-1:
				ipds_fns.append(fn)
			elif fn.find("_ipdsN.tmp")>-1:
				ipds_N_fns.append(fn)
			elif fn.find("_compN.tmp")>-1:
				comp_N_fns.append(fn)
			elif fn.find("_compkmers.tmp")>-1:
				comp_kmers_fns.append(fn)
			elif fn.find("_ipdskmers.tmp")>-1:
				ipds_kmers_fns.append(fn)

		labels_fns.sort()
		lengths_fns.sort()
		refs_fns.sort()
		readnames_fns.sort()
		strands_fns.sort()
		ipds_fns.sort()
		ipds_N_fns.sort()
		comp_N_fns.sort()
		
		cat_list_of_files(labels_fns,    os.path.join(self.opts.tmp, self.fns["read_labels"]), del_ins=False)
		cat_list_of_files(lengths_fns,   os.path.join(self.opts.tmp, self.fns["read_lengths"]), del_ins=False)
		cat_list_of_files(refs_fns,      os.path.join(self.opts.tmp, self.fns["read_refs"]), del_ins=False)
		cat_list_of_files(readnames_fns, os.path.join(self.opts.tmp, self.fns["read_names"]), del_ins=False)
		cat_list_of_files(strands_fns,   os.path.join(self.opts.tmp, self.fns["read_strands"]), del_ins=False)
		cat_list_of_files(ipds_fns,      os.path.join(self.opts.tmp, self.fns["read_SMp"]), del_ins=False)
		cat_list_of_files(ipds_N_fns,    os.path.join(self.opts.tmp, self.fns["read_SMp_counts"]), del_ins=False)
		cat_list_of_files(comp_N_fns,    os.path.join(self.opts.tmp, self.fns["read_comp"]), del_ins=False)
		shutil.copy(comp_kmers_fns[0],              os.path.join(self.opts.tmp, self.fns["read_comp_kmers"]))
		shutil.copy(ipds_kmers_fns[0],              os.path.join(self.opts.tmp, self.fns["read_SMp_kmers"]))

		for fn in comp_kmers_fns:
			os.remove(fn)

		for fn in ipds_kmers_fns:
			os.remove(fn)

	def combine_subreads_for_contig_level( self, cmph5_file ):
		readnames         = np.loadtxt(os.path.join(self.opts.tmp, self.fns["read_names"]),       dtype="str")
		read_refs         = np.loadtxt(os.path.join(self.opts.tmp, self.fns["read_refs"]),        dtype="str")
		reads_SMp         = np.loadtxt(os.path.join(self.opts.tmp, self.fns["read_SMp"]),         dtype="float")
		reads_SMp_counts  = np.loadtxt(os.path.join(self.opts.tmp, self.fns["read_SMp_counts"]),  dtype="int")
		reads_lengths     = np.loadtxt(os.path.join(self.opts.tmp, self.fns["read_lengths"]),     dtype="int")
		
		if len(reads_SMp.shape)==1:
			reads_SMp        = reads_SMp.reshape(reads_SMp.shape[0],1)
			reads_SMp_counts = reads_SMp_counts.reshape(reads_SMp_counts.shape[0],1)
		
		n_motifs  = reads_SMp.shape[1]
		ref_SCp   = {}
		ref_SCp_N = {}
		ref_bases = {}
		for ref in set(read_refs):
			idx            = read_refs==ref
			ref_bases[ref] = reads_lengths[idx].sum()
			ref_SCp[ref]   = []
			ref_SCp_N[ref] = []
			for j in range(n_motifs):
				motif_counts  = reads_SMp_counts[idx,j]
				motif_vals    = reads_SMp[idx,j]
				# Don't include data from reads without a motif instance
				motif_vals    = motif_vals[motif_counts>0]
				motif_counts  = motif_counts[motif_counts>0]
				if sum(motif_counts) > 0:
					contig_SCp   = sum(motif_vals * motif_counts) / sum(motif_counts)
					contig_SCp_N = sum(motif_counts)
				else:
					contig_SCp   = 0.0
					contig_SCp_N = 0
				ref_SCp[ref].append( contig_SCp )
				ref_SCp_N[ref].append( contig_SCp_N )

		def calc_contig_comps( contigs_fasta ):
			contigs_comp_strings = {}
			contig_fasta_lens    = {}
			for i,(name,contig_seq) in enumerate(self.fasta_iter(contigs_fasta)):
				contig_comp  = []
				contig_kmers = read_scanner.kmer_freq( "cmp", contig_seq, 0, self.opts )
				for kmer,count in contig_kmers.iteritems():
					kmer_normed_comp = math.log(float(count) / sum(contig_kmers.values()))
					contig_comp.append(kmer_normed_comp)
				contig_comp_str               = "\t".join(map(lambda x: str(round(x,4)), contig_comp))
				if name.find("|quiver")>-1:
					name = name.replace("|quiver", "") 
				refName                       = slugify(name)
				contigs_comp_strings[refName] = contig_comp_str
				contig_fasta_lens[refName]    = len(contig_seq)
			contigs_comp_strings["unknown"]   = "\t".join(map(lambda x: str(round(x,4)), [0.000] * len(contig_comp)))
			return contigs_comp_strings,contig_fasta_lens

		# Calculate composition profiles for all contigs in the contigs.fasta file
		contigs_comp_strings,self.contig_fasta_lens = calc_contig_comps( self.opts.contigs )

		f_ref_names       = open(os.path.join(self.opts.tmp, self.fns["contig_names"]),   "w")
		f_ref_SCp         = open(os.path.join(self.opts.tmp, self.fns["contig_SCp"]),     "w")
		f_ref_SCp_N       = open(os.path.join(self.opts.tmp, self.fns["contig_SCp_N"]),   "w")
		f_ref_comp        = open(os.path.join(self.opts.tmp, self.fns["contig_comp"]),    "w")
		f_ref_labels      = open(os.path.join(self.opts.tmp, self.fns["contig_labels"]),  "w")
		f_ref_covs        = open(os.path.join(self.opts.tmp, self.fns["contig_cov"]),     "w")
		f_ref_lens        = open(os.path.join(self.opts.tmp, self.fns["contig_lengths"]), "w")
		if self.opts.comp_only:
			ref_names         = contigs_comp_strings.keys()
			ref_names_str     = "\n".join(ref_names)
			f_ref_names.write("%s\n" % ref_names_str)
			for ref in contigs_comp_strings.keys():
				ref_SCp_str   = "0.000\t0.000\t0.000"
				ref_SCp_N_str = "0\t0\t0"
				f_ref_SCp.write(   "%s\n" % ref_SCp_str)
				f_ref_SCp_N.write( "%s\n" % ref_SCp_N_str)
				
				# Sometimes the contig name in cmp.h5 trims off the "|quiver" in "<unitig_N>|quiver"
				try:
					f_ref_comp.write(  "%s\n" % contigs_comp_strings[ref])
				except KeyError:
					# quiver_ref = slugify(ref+"|quiver")
					quiver_ref = slugify(ref)
					f_ref_comp.write(  "%s\n" % contigs_comp_strings[quiver_ref])
				
				f_ref_labels.write("%s\n" % self.opts.aln_fn_labels[cmph5_file])
				if self.opts.sam!=None:
					if ref=="unknown":
						coverage = 0
					else:
						coverage = float(ref_bases[ref]) / self.contig_fasta_lens[ref]
				else:
					coverage = float(ref_bases[ref]) / self.opts.aln_fn_contig_lens[cmph5_file][ref]
				f_ref_covs.write(  "%.2f\n" % coverage)
		else:
			ref_names         = ref_SCp.keys()
			ref_names_str     = "\n".join(ref_names)
			f_ref_names.write("%s\n" % ref_names_str)
			for ref in ref_names:
				ref_SCp_str   = "\t".join(map(lambda x: str(round(x,4)), ref_SCp[ref]))
				ref_SCp_N_str = "\t".join(map(lambda x: str(x), ref_SCp_N[ref]))
				f_ref_SCp.write(   "%s\n" % ref_SCp_str)
				f_ref_SCp_N.write( "%s\n" % ref_SCp_N_str)
				
				# Sometimes the contig name in cmp.h5 trims off the "|quiver" in "<unitig_N>|quiver"
				try:
					f_ref_comp.write(  "%s\n" % contigs_comp_strings[ref])
				except KeyError:
					# quiver_ref = slugify(ref+"|quiver")
					quiver_ref = slugify(ref)
					f_ref_comp.write(  "%s\n" % contigs_comp_strings[quiver_ref])
				
				f_ref_labels.write("%s\n" % self.opts.aln_fn_labels[cmph5_file])
				if self.opts.sam!=None:
					if ref=="unknown":
						coverage = 0
					else:
						coverage = float(ref_bases[ref]) / self.contig_fasta_lens[ref]
				else:
					coverage = float(ref_bases[ref]) / self.opts.aln_fn_contig_lens[cmph5_file][ref]
				f_ref_covs.write(  "%.2f\n" % coverage)
				f_ref_lens.write(  "%s\n" % self.opts.aln_fn_contig_lens[cmph5_file][ref])
		f_ref_names.close()
		f_ref_SCp.close()
		f_ref_SCp_N.close()
		f_ref_comp.close()
		f_ref_labels.close()
		f_ref_covs.close()
		f_ref_lens.close()

	def combine_contigs_for_bin_level( self ):
		"""
		If --cross_cov_bins is specified, aggregate IPD scores from contigs
		into bin-level scores based on their assignments from cross-coverage
		binning results. Recall that not all contigs will always have bin 
		assignments.
		"""

		bin_map = dict(np.loadtxt(self.opts.cross_cov_bins, delimiter=",", dtype="str"))

		contigs_names   = np.loadtxt(os.path.join(self.opts.tmp, self.fns["contig_names"]),   dtype="str")
		contigs_SCp     = np.loadtxt(os.path.join(self.opts.tmp, self.fns["contig_SCp"]),     dtype="float")
		contigs_SCp_N   = np.loadtxt(os.path.join(self.opts.tmp, self.fns["contig_SCp_N"]),   dtype="int")
		contigs_lengths = np.loadtxt(os.path.join(self.opts.tmp, self.fns["contig_lengths"]), dtype="int")
		
		if len(contigs_SCp.shape)==1:
			contigs_SCp   = contigs_SCp.reshape(contigs_SCp.shape[0],1)
			contigs_SCp_N = contigs_SCp_N.reshape(contigs_SCp_N.shape[0],1)

		# Ignore contigs without bin assignments
		binned_contigs_names   = []
		binned_contigs_SCp     = []
		binned_contigs_SCp_N   = []
		binned_contigs_lengths = []
		binned_contigs_bins    = []
		for i,name in enumerate(contigs_names):
			if bin_map.get(name):
				binned_contigs_names.append(name)
				binned_contigs_SCp.append(contigs_SCp[i,:])
				binned_contigs_SCp_N.append(contigs_SCp_N[i,:])
				binned_contigs_lengths.append(contigs_lengths[i])
				binned_contigs_bins.append(bin_map[name])
			else:
				pass

		binned_contigs_names   = np.array(binned_contigs_names)
		binned_contigs_SCp     = np.array(binned_contigs_SCp)
		binned_contigs_SCp_N   = np.array(binned_contigs_SCp_N)
		binned_contigs_lengths = np.array(binned_contigs_lengths)
		binned_contigs_bins    = np.array(binned_contigs_bins)

		n_motifs  = binned_contigs_SCp.shape[1]
		bins_SBp   = {}
		bins_SBp_N = {}
		bins_sizes = {}
		for bin_id in set(bin_map.values()):
			idx                = binned_contigs_bins==bin_id
			bins_sizes[bin_id] = binned_contigs_lengths[idx].sum()
			bins_SBp[bin_id]   = []
			bins_SBp_N[bin_id] = []
			for j in range(n_motifs):
				motif_counts  = binned_contigs_SCp_N[idx,j]
				motif_vals    = binned_contigs_SCp[idx,j]
				# Don't include data from contigs without a motif instance
				motif_vals    = motif_vals[motif_counts>0]
				motif_counts  = motif_counts[motif_counts>0]
				if sum(motif_counts) > 0:
					motif_SBp   = sum(motif_vals * motif_counts) / sum(motif_counts)
					motif_SBp_N = sum(motif_counts)
				else:
					motif_SBp   = 0.0
					motif_SBp_N = 0
				bins_SBp[bin_id].append( motif_SBp )
				bins_SBp_N[bin_id].append( motif_SBp_N )

		f_bin_names = open(self.bin_names_fn, "wb")
		f_bin_SBp   = open(self.bin_SCp_fn,   "wb")
		f_bin_SBp_N = open(self.bin_SCp_N_fn, "wb")
		f_bin_sizes = open(self.bin_sizes_fn, "wb")
		
		bin_names = bins_SBp.keys()
		bin_names.sort()

		for bin_id in bin_names:
			if bins_SBp.get(bin_id):
				bins_SBp_str   = "\t".join(map(lambda x: str(round(x,4)), bins_SBp[bin_id]))
				bins_SBp_N_str = "\t".join(map(lambda x: str(x), bins_SBp_N[bin_id]))
				f_bin_SBp.write(   "%s\n" % bins_SBp_str)
				f_bin_SBp_N.write( "%s\n" % bins_SBp_N_str)
				f_bin_sizes.write( "%s\n" % bins_sizes[bin_id])
				f_bin_names.write( "%s\n" % bin_id)
			else:
				# Did not collect data for that bin
				pass
		
		f_bin_names.close()
		f_bin_SBp.close()
		f_bin_SBp_N.close()
		f_bin_sizes.close()

	def combine_subread_data_across_bas_movies( self ):
		subreads_fns   = glob.glob( os.path.join(self.opts.tmp, "subreads_*.*.tmp") )
		labels_fns     = []
		lengths_fns    = []
		readnames_fns  = []
		ipds_fns       = []
		ipds_N_fns     = []
		comp_N_fns     = []
		comp_kmers_fns = []
		ipds_kmers_fns = []
		for fn in subreads_fns:
			if   fn.find("_labels.")>-1:
				labels_fns.append(fn)
			elif fn.find("_lengths.")>-1:
				lengths_fns.append(fn)
			elif fn.find("_names.")>-1:
				readnames_fns.append(fn)
			elif fn.find("_ipds.")>-1:
				ipds_fns.append(fn)
			elif fn.find("_ipdsN.")>-1:
				ipds_N_fns.append(fn)
			elif fn.find("_compN.")>-1:
				comp_N_fns.append(fn)
			elif fn.find("_compkmers.")>-1:
				comp_kmers_fns.append(fn)
			elif fn.find("_ipdskmers.")>-1:
				ipds_kmers_fns.append(fn)

		labels_fns.sort()
		lengths_fns.sort()
		readnames_fns.sort()
		ipds_fns.sort()
		ipds_N_fns.sort()
		comp_N_fns.sort()
		
		cat_list_of_files(labels_fns,    os.path.join(self.opts.tmp, "subreads_labels.tmp"))
		cat_list_of_files(lengths_fns,   os.path.join(self.opts.tmp, "subreads_lengths.tmp"))
		cat_list_of_files(readnames_fns, os.path.join(self.opts.tmp, "subreads_names.tmp"))
		cat_list_of_files(ipds_fns,      os.path.join(self.opts.tmp, "subreads_ipds.tmp"))
		cat_list_of_files(ipds_N_fns,    os.path.join(self.opts.tmp, "subreads_ipdsN.tmp"))
		cat_list_of_files(comp_N_fns,    os.path.join(self.opts.tmp, "subreads_compN.tmp"))
		shutil.copy(comp_kmers_fns[0],              os.path.join(self.opts.tmp, "subreads_compkmers.tmp"))
		shutil.copy(ipds_kmers_fns[0],              os.path.join(self.opts.tmp, "subreads_ipdskmers.tmp"))

		for fn in comp_kmers_fns:
			os.remove(fn)
		for fn in ipds_kmers_fns:
			os.remove(fn)

	def bas_combine_subreads_for_read_level( self, tmp_run=False ):
		subreads_fns = glob.glob( os.path.join(self.opts.tmp, "subreads_*.tmp") )
		for fn in subreads_fns:
			if   fn.find("_labels.tmp")>-1:
				labels_fn     = fn
			elif fn.find("_lengths.tmp")>-1:
				lengths_fn    = fn
			elif fn.find("_names.tmp")>-1:
				readnames_fn  = fn
			elif fn.find("_ipds.tmp")>-1:
				ipds_fn       = fn
			elif fn.find("_ipdsN.tmp")>-1:
				ipds_N_fn     = fn
			elif fn.find("_compN.tmp")>-1:
				comp_N_fn     = fn
			elif fn.find("_compkmers.tmp")>-1:
				comp_kmers_fn = fn
			elif fn.find("_ipdskmers.tmp")>-1:
				ipds_kmers_fn = fn

		readname_row_idx = defaultdict(list)
		for i,line in enumerate(open(readnames_fn, "r").xreadlines()):
			readname = line.strip()
			readname_row_idx[readname].append(i)
		n_subreads = i+1

		for line in open(ipds_kmers_fn).xreadlines():
			n_motifs = len(line.strip().split("\t"))

		sublengths = np.loadtxt(lengths_fn,    dtype="int")
		ipds       = np.loadtxt(ipds_fn,       dtype="float")
		ipds_N     = np.loadtxt(ipds_N_fn,     dtype="int")
		ipds_kmers = np.loadtxt(ipds_kmers_fn, dtype="string")
		comp_N     = np.loadtxt(comp_N_fn,     dtype="int")
		comp_kmers = np.loadtxt(comp_kmers_fn, dtype="string")
		labels     = np.loadtxt(labels_fn,     dtype="string")

		if n_subreads>1:
			if n_motifs==1:
				# Only one motif survived motif-filtering
				# Case #3
				ipds       = ipds.reshape(ipds.shape[0],1)
				ipds_N     = ipds_N.reshape(ipds_N.shape[0],1)
				ipds_kmers = ipds_kmers.reshape(1)
			elif n_motifs>1:
				# The ipd data is loaded properly in matrix form
				# Case #1
				pass
			elif n_motifs==0:
				pass
		elif n_subreads==1:
			sublengths = sublengths.reshape(1)
			comp_N     = comp_N.reshape(1,comp_N.shape[0])
			if n_motifs==1:
				# Case #4
				ipds       = ipds.reshape(1,1)
				ipds_N     = ipds_N.reshape(1,1)
				ipds_kmers = ipds_kmers.reshape(1)
			elif n_motifs>1:
				# Case #2
				ipds       = ipds.reshape(1,ipds.shape[0])
				ipds_N     = ipds_N.reshape(1,ipds_N.shape[0])
			elif n_motifs==0:
				pass
		
		if tmp_run:
			reads_names_fn      = os.path.join(self.opts.tmp, "read_names.tmp"     )
			reads_labels_fn     = os.path.join(self.opts.tmp, "read_labels.tmp"    )
			reads_lengths_fn    = os.path.join(self.opts.tmp, "read_lengths.tmp"   )
			reads_ipds_fn       = os.path.join(self.opts.tmp, "read_ipds.tmp"      )
			reads_ipds_N_fn     = os.path.join(self.opts.tmp, "read_ipdsN.tmp"    )
			reads_ipds_kmers_fn = os.path.join(self.opts.tmp, "read_ipdskmers.tmp")
			reads_comp_N_fn     = os.path.join(self.opts.tmp, "read_compN.tmp"    )
			reads_comp_kmers_fn = os.path.join(self.opts.tmp, "read_compkmers.tmp")
		else:
			reads_names_fn      = os.path.join(self.opts.tmp, self.fns["read_names"])
			reads_labels_fn     = os.path.join(self.opts.tmp, self.fns["read_labels"])
			reads_lengths_fn    = os.path.join(self.opts.tmp, self.fns["read_lengths"])
			reads_ipds_fn       = os.path.join(self.opts.tmp, self.fns["read_SMp"])
			reads_ipds_N_fn     = os.path.join(self.opts.tmp, self.fns["read_SMp_counts"])
			reads_ipds_kmers_fn = os.path.join(self.opts.tmp, self.fns["read_SMp_kmers"])
			reads_comp_N_fn     = os.path.join(self.opts.tmp, self.fns["read_comp"])
			reads_comp_kmers_fn = os.path.join(self.opts.tmp, self.fns["read_comp_kmers"])

		f_reads   = open(reads_names_fn,   "w")
		f_labels  = open(reads_labels_fn,  "w")
		f_lengths = open(reads_lengths_fn, "w")
		f_ipds    = open(reads_ipds_fn,    "w")
		f_ipds_N  = open(reads_ipds_N_fn,  "w")
		f_comp_N  = open(reads_comp_N_fn,  "w")
		for readname,row_idx in readname_row_idx.iteritems():
			f_reads.write(  "%s\n" % readname)
			f_labels.write( "%s\n" % labels[row_idx][0])
			comp_N_list  = comp_N[row_idx,:].sum(axis=0)
			ipds_list    = ipds[row_idx,:].mean(axis=0)
			ipds_N_list  = ipds_N[row_idx,:].sum(axis=0)

			# Normalize composition kmer counts
			normed_comp_N_list = map(lambda x: math.log( float(x)/sum(comp_N_list) ), comp_N_list)
			readlength         = sublengths[row_idx].sum()
			f_lengths.write( "%s\n" % readlength)
			f_ipds.write(    "%s\n" % "\t".join(map(lambda x: str(round(x,4)), ipds_list)))
			f_ipds_N.write(  "%s\n" % "\t".join(map(lambda x: str(x),          ipds_N_list)))
			f_comp_N.write(  "%s\n" % "\t".join(map(lambda x: str(round(x,4)), normed_comp_N_list)))
		
		shutil.copy(ipds_kmers_fn, reads_ipds_kmers_fn)
		shutil.copy(comp_kmers_fn, reads_comp_kmers_fn)
		f_reads.close()
		f_labels.close()
		f_lengths.close()
		f_ipds.close()
		f_ipds_N.close()
		f_comp_N.close()

		# Remove the subread-level barcodes for each contig
		for fn in subreads_fns:
			os.remove(fn)

	def get_read_refs_from_SAM( self ):
		read_refs = {}
		read_lens = {}
		samfile   = pysam.AlignmentFile(self.opts.sam, "rb")
		for k,aln in enumerate(samfile.fetch()):
			if aln.mapq == 254:
				read            = "/".join(aln.query_name.split("/")[:2])
				read            = "_".join(read.split("_")[1:])
				# ref             = aln.reference_name.split("|")[0]
				ref             = slugify(aln.reference_name)
				read_refs[read] = ref
				read_lens[read] = len(aln.seq)

		# Write the read_refs file to match the readnames file
		readnames = np.loadtxt(os.path.join( self.opts.tmp, self.fns["read_names"]), dtype="str")
		f_names   = open(os.path.join( self.opts.tmp, self.fns["read_refs"]),    "wb")
		f_lens    = open(os.path.join( self.opts.tmp, self.fns["read_lengths"]), "wb")
		for readname in readnames:
			try:
				f_names.write("%s\n" % read_refs[readname])
			except KeyError:
				f_names.write("unknown\n")
			try:
				f_lens.write("%s\n" % read_lens[readname])
			except KeyError:
				f_lens.write("0\n")
		f_names.close()
		f_lens.close()

	def launch_subprocs( self, input_file, N_reads, opts ):
		"""
		"""
		logging.debug("Creating tasks...")
		tasks   = multiprocessing.JoinableQueue()
		results = multiprocessing.Queue()
		logging.debug("Done.")

		if opts.aligned:
			reader     = openIndexedAlignmentFile(input_file)
			to_check   = reader
			entries    = range(len(to_check))
		else:
			reader     = BasH5Reader(input_file)
			if opts.bas_whitelist != None and not opts.control_run:
				logging.info("Intersecting with whitelist...")
				bas_whitelist = set(np.loadtxt(opts.bas_whitelist, dtype="str"))
				to_check = [z for z in reader if z.zmwName in bas_whitelist]
			else:
				to_check = reader
			# Filter on zmw metrics
			pre      = len(to_check)
			logging.info("Starting with %s reads..." % pre)
			to_check = [z for z in to_check if z.zmwMetric("Productivity")==1 and \
											   z.zmwMetric("ReadScore")>opts.minReadScore and \
											   z.zmwMetric("Pausiness")<opts.maxPausiness]
			post     = len(to_check)
			logging.info("Dropped %s reads due to poor zmw metrics (%s remain)" % ((pre-post),post))
			# Filter on read length 
			pre      = post
			to_check = [z for z in to_check if np.sum([len(sub) for sub in z.subreads])>=opts.readlength_min]
			post     = len(to_check)
			logging.info("Dropped %s reads < %s (%s remain)" % ((pre-post),opts.readlength_min, post))
			entries  = np.array([z.holeNumber for z in to_check])

		reader.close()
		if len(entries) <= opts.procs * 5:
			procs = 1
		else:
			procs = opts.procs

		logging.debug("Starting consumers...")
		consumers     = [ multiproc.Consumer(tasks, results) for i in xrange(procs) ]
		for w in consumers:
			w.start()
		logging.debug("Done.")

		num_jobs       = procs
		N_target_reads = {}
		reads_left     = N_reads
		procs_left     = procs

		for job in range(num_jobs):
			N_target_reads[job] = int(math.ceil(float(reads_left)/procs_left))
			reads_left -= N_target_reads[job]
			procs_left -= 1

		logging.debug("Partitioning %s into %s chunks for analysis..." % (input_file, num_jobs))
		chunksize      = int(math.ceil(float( len(entries)/procs )))
		logging.info("Querying %s reads using %s chunks of size %s..." % (len(to_check), procs, chunksize))
		entries_chunks = list(self.chunks( entries, chunksize ))
		for chunk_id in range(procs):
			n   = N_target_reads[chunk_id]
			# If --N_reads used, this ensures we touch as many contigs as possible
			idx = entries_chunks[chunk_id]
			np.random.shuffle(idx)
			if opts.aligned:
				tasks.put(cmph5_read.subread_motif_processor( input_file, chunk_id, idx, n, opts.motifs, opts.bi_motifs, opts) )
				logging.debug("...%s (%s alignments)" % (chunk_id, len(entries)))
			else:
				tasks.put(baxh5_read.subread_motif_processor( input_file, chunk_id, idx, n, opts.motifs, opts.bi_motifs, opts) )
				logging.debug("...%s (%s reads)" % (chunk_id, len(entries)))
		logging.debug("Done")
		
		# Add a poison pill for each consumer
		for i in xrange(num_jobs):
			tasks.put(None)

		# Wait for all of the tasks to finish
		tasks.join()
		
		# Start printing results
		logging.info("Combining results data from all chunks...")
		parallel_results = []
		while num_jobs:
			result = results.get()
			parallel_results.append(result)
			num_jobs -= 1
			logging.info("...%s/%s" % ((procs-num_jobs),procs))
		logging.info("Done.")
		return parallel_results


if __name__=="__main__":
	app     = mbinRunner()
	results = app.run()



import multiprocessing
import logging

class Consumer(multiprocessing.Process):
	def __init__(self, task_queue, result_queue):
		multiprocessing.Process.__init__(self)
		self.task_queue   = task_queue
		self.result_queue = result_queue

	def run(self):
		proc_name = self.name
		while True:
			next_task = self.task_queue.get()
			if next_task is None:
				# Poison pill means shutdown
				logging.debug("%s: Exiting" % proc_name)
				self.task_queue.task_done()
				break
			logging.debug("%s: Starting" % proc_name)
			answer = next_task()
			self.task_queue.task_done()
			self.result_queue.put(answer)
		return


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'pysam >= 0.10.0',
    'numpy >= 1.7.1, < 1.14',
    'h5py >= 2.0.1',
    'pbcore >= 0.9.4',
    'scipy >= 0.12.0',
    'biopython >= 1.6.1',
    'matplotlib >= 1.5.0'
]

setup_requirements = [
    # TODO(jbeaulaurier): put setup requirements (distutils extensions, etc.) here
    "nose",
]

test_requirements = [
    # TODO: put package test requirements here
]

setup(
    name='mbin',
    version='1.1.0',
    description="mBin: a methylation-based binning framework for metagenomic SMRT sequencing reads",
    long_description=readme + '\n\n' + history,
    author="John Beaulaurier",
    author_email='john.beaulaurier@gmail.com',
    url='https://github.com/fanglab/mbin',
    packages=find_packages(include=['mbin']),
    include_package_data=True,
    # data_files=[('', ['bin/bh_tsne'])],
    install_requires=requirements,
    license="CC BY-NC-SA 4.0 license",
    zip_safe=False,
    keywords='mbin methylation sequencing binning metagenomic',
    # scripts=["bin/mbin"],
    entry_points={"console_scripts" : ["buildcontrols = mbin.controls:launch", \
                                       "filtermotifs = mbin.motifs:launch", \
                                       "methylprofiles = mbin.profiles:launch", \
                                       "mapfeatures = mbin.visualize:launch", \
                                       ]},
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: Free for non-commercial use',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
    ],
    test_suite='nose.collector',
    tests_require=test_requirements,
    setup_requires=setup_requirements,
)



import os,sys
from collections import defaultdict
import numpy as np
import optparse
import logging
from pbcore.io.align.CmpH5IO import CmpH5Reader
from pbcore.io.BasH5IO import BasH5Reader
import glob
import numpy as np
import logging
import shutil
import pickle
import math
import mbin
import motif_tools
from Bio import SeqIO

def launch():
	opts, input_files, motifs_fn = __parseArgs()
	__initLog(opts)

	motifs           = np.loadtxt(motifs_fn, dtype="str", ndmin=1)
	motifs,not_found = find_motifs_in_control(opts, motifs)
	if len(not_found)>0:
		logging.warning("")
		logging.warning("  ******************** Important *********************")
		logging.warning("  Did not find %s motifs in %s:" % (len(not_found), opts.control_pkl_name))
		for nf in not_found:
			logging.warning("       %s" % nf)
		logging.warning("  These motif(s) will be removed from further analysis.")
		logging.warning("  These %s motifs will be kept:" % len(motifs))
		for m in motifs:
			logging.warning("       %s" % m)
		logging.warning("  ****************************************************")
		logging.warning("")
	else:
		logging.info("Found entries for all %s motifs in %s" % (len(motifs), opts.control_pkl_name))


	build_profiles(opts, input_files, motifs, motifs_fn)

	print >> sys.stderr, "mBin methylation profiling has finished running. See log for details."

def write_contig_features( mbinRunner, opts ):
	"""
	Gather contig-level data generated by the pipeline and write
	to output files.
	"""
	methyl_fn = "%scontig_methyl_features.txt" % opts.prefix
	motifs_fn = "%scontig_motif_counts.txt"    % opts.prefix
	others_fn = "%scontig_other_features.txt"  % opts.prefix

	logging.info("   %s\t -- Methylation profiles for assembled contigs" % methyl_fn)
	logging.info("   %s  \t -- Motif counts on assembled contigs" % motifs_fn)
	logging.info("   %s\t -- Coverage and length values, as well as %s-mer frequency vectors for assembled contigs" % (others_fn, opts.comp_kmer))

	f_methyl  = open(methyl_fn, "wb")
	f_motifs  = open(motifs_fn, "wb")
	f_others  = open(others_fn, "wb")

	ref_names  = np.loadtxt(os.path.join(opts.tmp, mbinRunner.fns["contig_names"]),   dtype="str", ndmin=1)
	ref_SCp    = np.loadtxt(os.path.join(opts.tmp, mbinRunner.fns["contig_SCp"]),     dtype="float", ndmin=2)
	ref_SCp_N  = np.loadtxt(os.path.join(opts.tmp, mbinRunner.fns["contig_SCp_N"]),   dtype="int", ndmin=2)
	ref_comp   = np.loadtxt(os.path.join(opts.tmp, mbinRunner.fns["contig_comp"]),    dtype="float", ndmin=2)
	ref_covs   = np.loadtxt(os.path.join(opts.tmp, mbinRunner.fns["contig_cov"]),     dtype="float", ndmin=1)
	ref_lens   = np.loadtxt(os.path.join(opts.tmp, mbinRunner.fns["contig_lengths"]), dtype="int", ndmin=1)
	
	# Write header
	motifs       = np.loadtxt(os.path.join(opts.tmp, mbinRunner.fns["read_SMp_kmers"]),  dtype="str", ndmin=1)
	kmers        = np.loadtxt(os.path.join(opts.tmp, mbinRunner.fns["read_comp_kmers"]), dtype="str", ndmin=1)
	motif_header = "\t".join(motifs)
	kmer_header  = "\t".join(kmers)
	f_methyl.write("contig\tlength\t%s\n" % motif_header)
	f_motifs.write("contig\tlength\t%s\n" % motif_header)
	f_others.write("contig\tlength\tcoverage\t%s\n" % kmer_header)

	for i,name in enumerate(ref_names):
		methyl_str = "\t".join(ref_SCp[i,:].astype("str"))
		f_methyl.write("%s\t%s\t%s\n" % (name, ref_lens[i], methyl_str))

		motifs_str = "\t".join(ref_SCp_N[i,:].astype("str"))
		f_motifs.write("%s\t%s\t%s\n" % (name, ref_lens[i], motifs_str))

		comp_str   = "\t".join(ref_comp[i,:].astype("str"))
		f_others.write("%s\t%s\t%.2f\t%s\n" % (name, ref_lens[i], ref_covs[i], comp_str))

	f_methyl.close()
	f_motifs.close()
	f_others.close()

def write_aligned_read_features( mbinRunner, opts ):
	"""
	If --aligned_read_barcodes is used, in addition to the contig-
	level features output, copy the data from the tmp directory into 
	alignment-level output files.
	"""
	methyl_fn = "%salign_methyl_features.txt" % opts.prefix
	motifs_fn = "%salign_motif_counts.txt"    % opts.prefix
	others_fn = "%salign_other_features.txt"  % opts.prefix

	logging.info("   %s\t -- Methylation profiles for aligned reads" % methyl_fn)
	logging.info("   %s  \t -- Motif counts on aligned reads" % motifs_fn)
	logging.info("   %s\t -- Length values, as well as %s-mer frequency vectors for aligned reads" % (others_fn, opts.comp_kmer))

	f_methyl  = open(methyl_fn, "wb")
	f_motifs  = open(motifs_fn, "wb")
	f_others  = open(others_fn, "wb")

	align_names = np.loadtxt(os.path.join(opts.tmp, mbinRunner.fns["read_names"]),      dtype="str",   ndmin=1)
	align_SMp   = np.loadtxt(os.path.join(opts.tmp, mbinRunner.fns["read_SMp"]),        dtype="float", ndmin=2)
	align_SMp_N = np.loadtxt(os.path.join(opts.tmp, mbinRunner.fns["read_SMp_counts"]), dtype="int",   ndmin=2)
	align_comp  = np.loadtxt(os.path.join(opts.tmp, mbinRunner.fns["read_comp"]),       dtype="float", ndmin=2)
	align_lens  = np.loadtxt(os.path.join(opts.tmp, mbinRunner.fns["read_lengths"]),    dtype="int",   ndmin=1)

	# Write header
	motifs       = np.loadtxt(os.path.join(opts.tmp, mbinRunner.fns["read_SMp_kmers"]),  dtype="str", ndmin=1)
	kmers        = np.loadtxt(os.path.join(opts.tmp, mbinRunner.fns["read_comp_kmers"]), dtype="str", ndmin=1)
	motif_header = "\t".join(motifs)
	kmer_header  = "\t".join(kmers)
	f_methyl.write("read\tlength\t%s\n" % motif_header)
	f_motifs.write("read\tlength\t%s\n" % motif_header)
	f_others.write("read\tlength\t%s\n" % kmer_header)

	for i,name in enumerate(align_names):
		methyl_str = "\t".join(align_SMp[i,:].astype("str"))
		f_methyl.write("%s\t%s\t%s\n" % (name, align_lens[i], methyl_str))

		motifs_str = "\t".join(align_SMp_N[i,:].astype("str"))
		f_motifs.write("%s\t%s\t%s\n" % (name, align_lens[i], motifs_str))

		comp_str   = "\t".join(align_comp[i,:].astype("str"))
		f_others.write("%s\t%s\t%s\n" % (name, align_lens[i], comp_str))

	f_methyl.close()
	f_motifs.close()
	f_others.close()

def write_unaligned_read_features( mbinRunner, opts ):
	"""
	Gather unaligned read-level data generated by the pipeline and 
	write to output files.
	"""
	methyl_fn = "%sread_methyl_features.txt" % opts.prefix
	motifs_fn = "%sread_motif_counts.txt"    % opts.prefix
	others_fn = "%sread_other_features.txt"  % opts.prefix

	logging.info("   %s\t -- Methylation profiles for unaligned reads" % methyl_fn)
	logging.info("   %s  \t -- Motif counts for unaligned reads" % motifs_fn)
	logging.info("   %s\t -- Length values, as well as %s-mer frequency vectors for unaligned reads" % (others_fn, opts.comp_kmer))

	f_methyl  = open(methyl_fn, "wb")
	f_motifs  = open(motifs_fn, "wb")
	f_others  = open(others_fn, "wb")

	read_names = np.loadtxt(os.path.join(opts.tmp, mbinRunner.fns["read_names"]),      dtype="str",   ndmin=1)
	read_SMp   = np.loadtxt(os.path.join(opts.tmp, mbinRunner.fns["read_SMp"]),        dtype="float", ndmin=2)
	read_SMp_N = np.loadtxt(os.path.join(opts.tmp, mbinRunner.fns["read_SMp_counts"]), dtype="int",   ndmin=2)
	read_comp  = np.loadtxt(os.path.join(opts.tmp, mbinRunner.fns["read_comp"]),       dtype="float", ndmin=2)
	read_lens  = np.loadtxt(os.path.join(opts.tmp, mbinRunner.fns["read_lengths"]),    dtype="int",   ndmin=1)

	# Write header
	motifs       = np.loadtxt(os.path.join(opts.tmp, mbinRunner.fns["read_SMp_kmers"]),  dtype="str", ndmin=1)
	kmers        = np.loadtxt(os.path.join(opts.tmp, mbinRunner.fns["read_comp_kmers"]), dtype="str", ndmin=1)
	motif_header = "\t".join(motifs)
	kmer_header  = "\t".join(kmers)
	f_methyl.write("read\tlength\t%s\n" % motif_header)
	f_motifs.write("read\tlength\t%s\n" % motif_header)
	f_others.write("read\tlength\t%s\n" % kmer_header)

	for i,name in enumerate(read_names):
		methyl_str = "\t".join(read_SMp[i,:].astype("str"))
		f_methyl.write("%s\t%s\t%s\n" % (name, read_lens[i], methyl_str))

		motifs_str = "\t".join(read_SMp_N[i,:].astype("str"))
		f_motifs.write("%s\t%s\t%s\n" % (name, read_lens[i], motifs_str))

		comp_str   = "\t".join(read_comp[i,:].astype("str"))
		f_others.write("%s\t%s\t%s\n" % (name, read_lens[i], comp_str))

	f_methyl.close()
	f_motifs.close()
	f_others.close()

def find_motifs_in_control(opts, motifs):
	"""
	Make sure all specified motifs have control values in the control
	dictionary (specified by --control_pkl_name). Remove any motifs 
	that do not have control values.
	"""
	control_d    = pickle.load( open(opts.control_pkl_name,"rb" ) )
	found_motifs = []
	not_found    = []
	for m in motifs:
		if control_d.get(m):
			found_motifs.append(m)
		else:
			not_found.append(m)

	return np.array(found_motifs), not_found

def combine_subreads_for_read_level( tup ):
	input_file   = tup[0]
	contig       = tup[1]
	tmp          = tup[2]
	aln_fn_labels    = tup[3]
	j            = tup[4]
	N_contigs    = tup[5]
	logging.info("...contig %s (%s/%s)" % (contig, j, N_contigs))
	contig_fns = glob.glob( os.path.join(tmp, "%s_*.tmp" % contig) )
	for fn in contig_fns:
		if   fn.find("_labels.tmp")>-1:
			labels_fn     = fn
		elif fn.find("_lengths.tmp")>-1:
			lengths_fn    = fn
		elif fn.find("_readnames.tmp")>-1:
			readnames_fn  = fn
		elif fn.find("_ipds.tmp")>-1:
			ipds_fn       = fn
		elif fn.find("_ipdsN.tmp")>-1:
			ipds_N_fn     = fn
		elif fn.find("_compN.tmp")>-1:
			comp_N_fn     = fn
		elif fn.find("_compkmers.tmp")>-1:
			comp_kmers_fn = fn
		elif fn.find("_ipdskmers.tmp")>-1:
			ipds_kmers_fn = fn
		elif fn.find("_strand.tmp")>-1:
			strand_fn     = fn

	readname_row_idx = defaultdict(list)
	for i,line in enumerate(open(readnames_fn, "r").xreadlines()):
		readname = line.strip()
		readname_row_idx[readname].append(i)
	n_subreads = i+1

	for line in open(ipds_kmers_fn).xreadlines():
		n_motifs = len(line.strip().split("\t"))

	sublengths = np.loadtxt(lengths_fn,    dtype="int")
	ipds       = np.loadtxt(ipds_fn,       dtype="float")
	ipds_N     = np.loadtxt(ipds_N_fn,     dtype="int")
	ipds_kmers = np.loadtxt(ipds_kmers_fn, dtype="string")
	comp_N     = np.loadtxt(comp_N_fn,     dtype="int")
	comp_kmers = np.loadtxt(comp_kmers_fn, dtype="string")
	strands    = np.loadtxt(strand_fn,     dtype="int")

	if n_subreads>1:
		if n_motifs==1:
			# Only one motif survived motif-filtering
			# Case #3
			ipds       = ipds.reshape(ipds.shape[0],1)
			ipds_N     = ipds_N.reshape(ipds_N.shape[0],1)
			ipds_kmers = ipds_kmers.reshape(1)
		elif n_motifs>1:
			# The ipd data is loaded properly in matrix form
			# Case #1
			pass
		elif n_motifs==0:
			pass
	elif n_subreads==1:
		sublengths = sublengths.reshape(1)
		strands    = strands.reshape(1)
		comp_N     = comp_N.reshape(1,comp_N.shape[0])
		if n_motifs==1:
			# Case #4
			ipds       = ipds.reshape(1,1)
			ipds_N     = ipds_N.reshape(1,1)
			ipds_kmers = ipds_kmers.reshape(1)
		elif n_motifs>1:
			# Case #2
			ipds       = ipds.reshape(1,ipds.shape[0])
			ipds_N     = ipds_N.reshape(1,ipds_N.shape[0])
		elif n_motifs==0:
			pass
		
	reads_names_fn      = os.path.join(tmp, "%s_read_names.tmp"     % contig)
	reads_labels_fn     = os.path.join(tmp, "%s_read_labels.tmp"    % contig)
	reads_lengths_fn    = os.path.join(tmp, "%s_read_lengths.tmp"   % contig)
	reads_contig_fn     = os.path.join(tmp, "%s_read_contig.tmp"    % contig)
	reads_ipds_fn       = os.path.join(tmp, "%s_read_ipds.tmp"      % contig)
	reads_ipds_N_fn     = os.path.join(tmp, "%s_read_ipdsN.tmp"     % contig)
	reads_ipds_kmers_fn = os.path.join(tmp, "%s_read_ipdskmers.tmp" % contig)
	reads_comp_N_fn     = os.path.join(tmp, "%s_read_compN.tmp"     % contig)
	reads_comp_kmers_fn = os.path.join(tmp, "%s_read_compkmers.tmp" % contig)
	reads_strands_fn    = os.path.join(tmp, "%s_read_strands.tmp"   % contig)

	f_reads      = open(reads_names_fn,   "w")
	f_labels     = open(reads_labels_fn,  "w")
	f_lengths    = open(reads_lengths_fn, "w")
	f_contig     = open(reads_contig_fn,  "w")
	f_ipds       = open(reads_ipds_fn,    "w")
	f_ipds_N     = open(reads_ipds_N_fn,  "w")
	f_comp_N     = open(reads_comp_N_fn,  "w")
	f_strands    = open(reads_strands_fn, "w")
	label        = aln_fn_labels[input_file]
	for readname,row_idx in readname_row_idx.iteritems():
		f_reads.write(  "%s\n" % readname)
		f_labels.write( "%s\n" % label)
		f_contig.write( "%s\n" % contig)
		comp_N_list  = comp_N[row_idx,:].sum(axis=0)
		strands_list = strands[row_idx]
		# ipds_list   = ipds[row_idx,:].mean(axis=0)
		ipds_N_list  = ipds_N[row_idx,:].sum(axis=0)
		ipds_list    = []
		for k in range(n_motifs):
			read_motifs_N = np.sum(ipds_N[row_idx,k])
			read_ipds_sum = np.sum(ipds[row_idx,k] * ipds_N[row_idx,k])
			if read_motifs_N > 0:
				motif_mean = read_ipds_sum / read_motifs_N
			else:
				motif_mean = 0.0
			ipds_list.append(motif_mean)

		# Normalize composition kmer counts
		normed_comp_N_list = map(lambda x: math.log( float(x)/sum(comp_N_list) ), comp_N_list)
		readlength         = sublengths[row_idx].sum()
		f_lengths.write( "%s\n" % readlength)
		f_ipds.write(    "%s\n" % "\t".join(map(lambda x: str(round(x,4)), ipds_list)))
		f_ipds_N.write(  "%s\n" % "\t".join(map(lambda x: str(x),          ipds_N_list)))
		f_comp_N.write(  "%s\n" % "\t".join(map(lambda x: str(round(x,4)), normed_comp_N_list)))
		f_strands.write( "%s\n" %  ",".join(map(lambda x: str(x),          strands_list)))
	
	shutil.copy(ipds_kmers_fn, os.path.join(tmp, "%s_read_ipdskmers.tmp" % contig))
	shutil.copy(comp_kmers_fn, os.path.join(tmp, "%s_read_compkmers.tmp" % contig))
	f_reads.close()
	f_labels.close()
	f_lengths.close()
	f_contig.close()
	f_ipds.close()
	f_ipds_N.close()
	f_comp_N.close()
	f_strands.close()

	# Remove the subread-level barcodes for each contig
	for fn in contig_fns:
		os.remove(fn)

def build_profiles(opts, input_files, motifs, motifs_fn):
	"""
	Build profiles of methylation scores for the desired motifs
	across the sequences, either contigs or unaligned reads.
	"""
	if os.path.exists(opts.tmp):
		shutil.rmtree(opts.tmp)
	os.mkdir(opts.tmp)

	opts.motifs_file = motifs_fn
	opts.motifs      = motifs
	opts.bi_motifs   = None

	logging.info("Building methylation profiles using %s motifs..." % len(opts.motifs))
	to_del = glob.glob( os.path.join(opts.tmp, "*") )
	for fn in to_del:
		os.remove(fn)

	mbinRunner     = mbin.mbinRunner(opts)
	##################################################
	# Launch analysis of <N_reads> for motif filtering
	##################################################
	for i,input_file in enumerate(input_files):
		logging.info("Creating %s barcodes (%s motifs) from %s..." % (opts.N_reads, len(opts.motifs), input_file))
		mbinRunner.launch_data_loader( input_file, opts.N_reads, i, opts )

		if opts.input_ftype=="cmp" or opts.input_ftype=="bam":
			logging.info("Combining aligned subread-level barcodes to get read-level barcodes from each contig...")
			contig_labels_fns = glob.glob( os.path.join(opts.tmp, "*_labels.tmp") )
			contigs           = map(lambda x: os.path.basename(x).split("_labels.tmp")[0], contig_labels_fns)
			args              = [ (input_file, contig, opts.tmp, opts.aln_fn_labels, i, len(contigs)) for i,contig in enumerate(contigs)]
			results           = mbin.launch_pool( opts.procs, combine_subreads_for_read_level, args )

			logging.info("Combining read-level barcodes from all contigs...")
			mbinRunner.combine_read_level_barcodes_across_contigs()
			logging.info("Done.")

			logging.info("Creating contig-level barcodes (%s motifs) from %s..." % (len(opts.motifs), input_file))
			mbinRunner.combine_subreads_for_contig_level( input_file )
			logging.info("Done.")
			n_contigs = len(np.loadtxt(os.path.join(opts.tmp, mbinRunner.fns["contig_names"]), dtype="str", ndmin=1))

			if opts.cross_cov_bins!=None:
				logging.info("Creating bin-level barcodes (%s motifs) using %s..." % (len(opts.motifs), opts.cross_cov_bins))
				mbinRunner.combine_contigs_for_bin_level()
				logging.info("Done.")

	if opts.input_ftype=="bas":
		# Combine subread data across multiple movies
		logging.info("Combining subread data across all movies...")
		results = mbinRunner.combine_subread_data_across_bas_movies()
		logging.info("Done.")
		# Combine movie-merged subreads to get read-level barcodes
		logging.info("Combining subreads to get read-level barcodes...")
		results = mbinRunner.bas_combine_subreads_for_read_level()
		logging.info("Done.")

		if opts.sam!=None:
			logging.info("Writing read-contig assignments based on %s..." % opts.sam)
			mbinRunner.get_read_refs_from_SAM()
			logging.info("Done.")
			for i,input_file in enumerate(input_files):
				logging.info("Creating contig-level barcodes (%s motifs) from %s..." % (len(opts.motifs), input_file))
				mbinRunner.combine_subreads_for_contig_level( input_file )
				logging.info("Done.")
			n_contigs = len(np.loadtxt(os.path.join(opts.tmp, mbinRunner.fns["contig_names"]), dtype="str", ndmin=1))

	logging.info("Writing output files:")

	if opts.input_ftype=="cmp" or opts.input_ftype=="bam":
		write_contig_features(mbinRunner, opts)
		if opts.aligned_read_barcodes:
			write_aligned_read_features( mbinRunner, opts )
	elif opts.input_ftype=="bas":
		write_unaligned_read_features( mbinRunner, opts )

	logging.info("Cleaning up temp files from methylation profiling...")
	shutil.rmtree(opts.tmp)
	logging.info("Pipeline finished.")

def __parseArgs():
	"""Handle command line argument parsing"""

	usage = """%prog [--help] [options] [input_seqs] [motifs_file]

	methylprofiles compiles the methylation profiles (for contigs or reads)
	across the motifs specified in the arguments. The methylation profiles 
	can be constructed using either native reads aligned to assembled contigs 
	(*.bam or *.cmp.h5) or unaligned reads (*.bas.h5), the latter of which can 
	be supplied as a single bas.h5 file or a FOFN containing multiple *.bas.h5 
	files. Unaligned BAM files are not currently supported.

	The motifs specified in the motifs file must be line-separated and
	indicate the methylated position in the motif using a 0-based index. For 
	instance, GATC-1 indicates that the adenine position is methylated, while 
	CATAG-3	says that the second adenine in the motif is methylated. This file
	would look as follows:

	GATC-1
	CATAG-3

	Two files will be output from methylprofiles (and serve as input to the 
	routine mapfeatures): 
	   (1) <prefix>_<seq>_methyl_features.txt
	   (2) <prefix>_<seq>_other_features.txt

	<prefix> is defined by --prefix and <seq> is the sequence data type: contig, 
	align, or read.


	Usage examples:

	### Using BAM of aligned reads as input and motifs specified in motifs.txt ###

	methylprofiles -i --contigs=reference.fasta aligned_reads.bam motifs.txt
	

	### Using *.cmp.h5 of aligned reads as input and motifs specified in motifs.txt ###

	methylprofiles -i --contigs=reference.fasta aligned_reads.cmp.h5 motifs.txt


	### Using unaligned reads in a bas.h5 file ###

	methylprofiles -i m12345.bas.h5 motifs.txt


	### Using a FOFN of multiple bas.h5 files ###

	methylprofiles -i bas.h5.fofn motifs.txt
	"""

	parser = optparse.OptionParser( usage=usage, description=__doc__ )

	parser.add_option( "-d", "--debug", action="store_true", help="Increase verbosity of logging" )

	parser.add_option( "-i", "--info", action="store_true", help="Add basic logging" )

	parser.add_option( "--logFile", type="str", help="Write logging to file [log.controls]" )
	
	parser.add_option( "--prefix", type="str", help="Prefix to use for output files [None]" )

	parser.add_option( "--tmp", type="str", help="Directory where numerous temporary files will be written [profiles_tmp]" )
	
	parser.add_option( "--contigs", type="str", help="Fasta file containing entries for the assembled contigs [None]" )

	parser.add_option( "--minReadScore", type="float", help="Min read score of an unaligned read [0.0]" )

	parser.add_option( "--maxPausiness", type="float", help="Max pausiness value of an unaligned read [1000]" )

	parser.add_option( "--minQV", type="float", help="If base has QV < minQV, do not include [0]" )
	
	parser.add_option( "--subreadlength_min", type="int", help="Minimum subread length to include for analysis [100]" )

	parser.add_option( "--readlength_min", type="int", help="Minimum read length to include for analysis [100]" )

	parser.add_option( "--minContigLength", type="int", help="Min length of contig to consider [10000]" )
	
	parser.add_option( "--comp_kmer", type="int", help="Kmer size to use for sequence composition measurements [5]" )
	
	parser.add_option( "--aligned_read_barcodes", action="store_true", help="Also output features for individual aligned reads, not just contigs (requires cmp.h5 input) [False]" )

	parser.add_option( "--minAcc", type="float", help="Min subread accuracy of read [0.8]" )

	parser.add_option( "--minMapQV", type="float", help="Min mapping QV of aligned read [240]" )

	parser.add_option( "--procs", type="int", help="Number of processors to use [4]" )

	parser.add_option( "--N_reads", type="int", help="Number of qualifying reads to include (from each bas.h5 if input is FOFN of bas.h5 files) in analysis [1000000000]" )
	
	parser.add_option( "--control_pkl_name", type="str", help="Filename to save control IPD data from WGA sequencing [control_ipds.pkl]" )
	
	parser.add_option( "--no_subtract_control", action="store_true", help="Subtract control IPDs in final calculations [True]" )

	parser.add_option( "--cross_cov_bins", type="str", help="Path to file containing binning results from CONCOCT. Will use to improve motif discovery. Only works with contig-level analysis (cmp.h5 input) inputs. File format should be '<contig_name>,<bin_id>' [None]" )

	parser.set_defaults( logFile="log.methylprofiles",         \
						 info=False,                           \
						 debug=False,                          \
						 prefix="",                            \
						 tmp="profiles_tmp",                   \
						 contigs=None,                         \
						 minQV=0,                              \
						 subreadlength_min=100,                \
						 readlength_min=100,                   \
						 minContigLength=10000,                \
						 minReadScore=0.0,                     \
						 maxPausiness=1000,                    \
						 comp_kmer=5,                          \
						 aligned_read_barcodes=False,          \
						 minAcc=0.8,                           \
						 minMapQV=240,                         \
						 procs=4,                              \
						 N_reads=1000000000,                   \
						 control_pkl_name="control_ipds.pkl",  \
						 no_subtract_control=False,            \
						 cross_cov_bins=None,                  \
						 )

	opts, args = parser.parse_args( )

	input_files, motifs_fn  = __check_input( opts, args, parser )

	opts.comp_only     = False
	opts.sam           = None
	opts.skip_motifs   = None
	opts.bas_whitelist = None

	if opts.no_subtract_control:
		opts.subtract_control = False
	else:
		opts.subtract_control = True

	if opts.prefix != "":
		opts.prefix+="_"
	
	opts.control_pkl_name = os.path.abspath(opts.control_pkl_name)

	if not os.path.exists(opts.control_pkl_name):
		parser.error("Can't find control IPDs pickle file: %s" % opts.control_pkl_name)

	return opts,input_files,motifs_fn

def __initLog( opts ):
	"""Sets up logging based on command line arguments. Allows for three levels of logging:
	logging.error( ): always emitted
	logging.info( ) : emitted with --info or --debug
	logging.debug( ): only with --debug"""

	if os.path.exists(opts.logFile):
		os.remove(opts.logFile)

	logLevel = logging.DEBUG if opts.debug \
				else logging.INFO if opts.info \
				else logging.ERROR

	logger = logging.getLogger("")
	logger.setLevel(logLevel)
	
	# create file handler which logs even debug messages
	fh = logging.FileHandler(opts.logFile)
	fh.setLevel(logLevel)
	
	# create console handler with a higher log level
	ch = logging.StreamHandler()
	ch.setLevel(logLevel)
	
	# create formatter and add it to the handlers
	logFormat = "%(asctime)s [%(levelname)s] %(message)s"
	formatter = logging.Formatter(logFormat, "%Y-%m-%d %H:%M:%S")
	ch.setFormatter(formatter)
	fh.setFormatter(formatter)
	
	# add the handlers to logger
	logger.addHandler(ch)
	logger.addHandler(fh)

def __check_input( opts, args, parser ):
	"""
	Verify that the input format looks OK and assess the input type.
	"""
	if len(args)!=2:
		print "ERROR -- expecting two arguments:"
		print "    (1) input file (bam, cmp.h5, bas.h5, or FOFN of bas.h5 files)"
		print "    (2) file containing the motifs to analyze, separated by newlines, e.g."
		print "          GATC-1"
		print "          CATG-1"
		print "          CATAG-3"
		print ""
		sys.exit()

	input_files, opts = mbin.check_input_ftype(opts, args, parser)

	if opts.input_ftype=="bas" and opts.cross_cov_bins!=None:
		parser.error("Use of the --cross_cov_bins option is not compatible with bas.h5 inputs!")

	if opts.input_ftype=="cmp" or opts.input_ftype=="bam":
		try:
			for entry in SeqIO.parse(opts.contigs, "fasta"):
				x = entry.seq
				y = entry.id
		except:
			parser.error("Please make sure the --contigs input is a valid fasta file.")

	motifs_fn          = os.path.abspath(args[1])

	if not os.path.exists(motifs_fn):
		parser.error("Can't find file of motifs to include in methylation profile: %s" % motifs_fn)

	return input_files, motifs_fn

if __name__ == "__main__":
	main()